using System.IO;
using System.Threading.Tasks;
using BatchProcess_GroupF_06.ViewModels;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.Http;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using BatchProcess_GroupF_06.Helper;
using System.Collections.Generic;
using System.Linq;
using BatchProcess_GroupF_06.Models;
using System.Net.Http;
using System.IO.Compression;
using System;
using iTextSharp.text.pdf;
using CommonLibrary.Models;
using iTextSharp.text;
using BatchProcess_GroupF_06.EIF;
using System.Net.Http.Headers;
using CommonLibrary.Log;
using CommonLibrary.Log.Models.Batch;

namespace BatchProcess_GroupF_06
{
    public class BID0018
    {

        private const eBatchAPIs batchID = eBatchAPIs.BID0018;

        private ILogger_Common_Batch logger;

        public BID0018(ILogger_Common_Batch logger)
        {
            this.logger = logger;
        }

        /// <summary>
        /// This batch returns the article text (PDF) file in response to Http access from the external system.
        /// </summary>
        /// <param name="req"></param>
        /// <param name="log"></param>
        /// <returns></returns>
        
        [FunctionName("BID0018")]
        public async Task<HttpResponseMessage> Run([HttpTrigger(AuthorizationLevel.Anonymous, "get", "post", Route = "v1/download/mc_articles.pdf")] HttpRequest req, ILogger log)
        {
            try
            {
                logger.Trace(new MethodInBatch(batchID));
                var result = new HttpResponseMessage(System.Net.HttpStatusCode.OK);
                var BID0018_InputJson = req.ReadAsStringAsync().Result;
                var BID0018_DeserializeInput = JsonConvert.DeserializeObject<BID0018_ViewModel>(BID0018_InputJson);
                
                var validationResults = new List<System.ComponentModel.DataAnnotations.ValidationResult>();

                if (!Isvalid_Model.IsValid(BID0018_DeserializeInput, validationResults))
                {
                    result = new HttpResponseMessage(System.Net.HttpStatusCode.InternalServerError);
                    result.Content = new StringContent(System.Net.HttpStatusCode.InternalServerError.ToString());
                    return result;
                }
                  
                
                string User_id = BID0018_DeserializeInput.api_id.ToString();
                string User_pass = BID0018_DeserializeInput.api_pass.ToString();
                string ContractCode = BID0018_DeserializeInput.contract_code.ToString();               
                var DbContent = DB_config.db_conection();

                string hash_user_pass = BatchCommonUtil.PasswordHashing(User_pass);

                string Revice_data = JsonConvert.SerializeObject(BID0018_DeserializeInput);
                logger.Trace(new ParameterSendReceiveMethod(batchID, parameterType.Receive, Revice_data));
                
                var BID0018_IsValidUser = from u in DbContent.User
                                          join c in DbContent.Contract on u.ContractId equals c.ContractId
                                          join ug in DbContent.UserGroup on u.UserGroupId equals ug.UserGroupId
                                          where c.ContractCode == ContractCode && u.UserId == User_id && u.Password == hash_user_pass
                                          && u.DelFlag == false && c.DelFlag == false && ug.DelFlag == false 
                                          select new
                                          {
                                              contract_id = u.ContractId,
                                              user_group_name = ug.UserGroupName,
                                              demand_id = ug.DemandId
                                          };
                if (BID0018_IsValidUser.Any())
                {
                    string BID0018_SessionID =Guid.NewGuid().ToString("D");

                    var Get_MC_UserPass = (from sl in DbContent.ServiceLinkage
                                           join c in DbContent.Contract on sl.ContractId equals c.ContractId
                                           where sl.DataSection == Data_Section.MC && c.ContractCode == ContractCode
                                           && sl.DelFlag == false && c.DelFlag == false 
                                           select new
                                           {
                                               Api_id = sl.ApiId,
                                               API_pass = sl.ApiPass,
                                               contract_id = sl.ContractId
                                           }).ToList();

                    string UserGroupName = BID0018_IsValidUser.First().user_group_name.ToString();
                    var ContractId = BID0018_IsValidUser.First().contract_id;
                    // Add record to authentication_log
                    AuthenticationLog AuthLog = new AuthenticationLog
                    {
                        ContractId = ContractId,
                        UserId = User_id,
                        SessionId = BID0018_SessionID,
                        LoginDatetime = await BatchCommonUtil.Get_Current_DateTime(),
                        FunctionSection = Function_Section.BID0018,
                        DemandId = BID0018_IsValidUser.First().demand_id,
                        UserGroupName = UserGroupName

                    };
                    DbContent.AuthenticationLog.Add(AuthLog);
                    DbContent.SaveChanges();
                    //----------------------------------------------------------------                    
                    var ArticleId_Count = BID0018_DeserializeInput.article_id.Count();
                    if (ArticleId_Count > 0)
                    {
                        byte[] fileBytes = null;
                        using (MemoryStream zipStream = new MemoryStream())
                        {
                            using (ZipArchive zip = new ZipArchive(zipStream, ZipArchiveMode.Create, true))
                            {
                                for (int i = 0; i < ArticleId_Count; i++)
                                {
                                    var check_article = (from mc in DbContent.McArticle
                                                         join ta in DbContent.ThemeArticle on new { mc.ArticleId, mc.ProcessDate } equals new { ta.ArticleId, ta.ProcessDate }
                                                         where mc.ArticleId == BID0018_DeserializeInput.article_id[i].ToString()
                                                         && mc.DispStatus == 0 && ta.ContractId== ContractId
                                                         && mc.DelFlag == false
                                                         select mc).ToList();
                                    if (check_article.Any())
                                    {
                                        IN_WAPI008_PDF wAPI008_PDF = new IN_WAPI008_PDF
                                        {
                                            ac = BID0018_DeserializeInput.article_id[i].ToString(),
                                            dmsid = Get_MC_UserPass.First().Api_id.ToString(),
                                            password = Get_MC_UserPass.First().API_pass.ToString(),
                                            mcuserid = User_id.ToString()
                                        };

                                        string send_data = JsonConvert.SerializeObject(wAPI008_PDF);
                                        logger.Trace(new ParameterSendReceiveMethod(batchID, parameterType.Send, send_data));
                                        
                                        OP_MC_WAPI008_PDF BID0018_Response = await AID9026.Get_MC_WAPI008_PDF(wAPI008_PDF);
                                        if (BID0018_Response != null)
                                        {
                                            if (BID0018_Response.ExternalAPIResonse.IsSuccessResponse)
                                            {
                                                byte[] pdf = Convert.FromBase64String(BID0018_Response.pdf);

                                                MemoryStream BID0018_MS_Output = new MemoryStream();
                                                BID0018_MS_Output.Write(pdf, 0, pdf.Length);
                                                BID0018_MS_Output.Seek(0, SeekOrigin.Begin);
                                                byte[] bytes = BID0018_MS_Output.ToArray();
                                                Font blackFont = FontFactory.GetFont("Arial", 12, Font.NORMAL, BaseColor.BLACK);
                                                using (MemoryStream stream = new MemoryStream())
                                                {
                                                    PdfReader reader = new PdfReader(bytes);
                                                    using (PdfStamper stamper = new PdfStamper(reader, stream))
                                                    {
                                                        stamper.Writer.CloseStream = false;
                                                        int pages = reader.NumberOfPages;
                                                        for (int j = 1; j <= pages; j++)
                                                        {
                                                            ColumnText.ShowTextAligned(stamper.GetUnderContent(j), Element.ALIGN_RIGHT, new Phrase(j.ToString(), blackFont), 568f, 15f, 0);
                                                        }
                                                        stamper.Close();
                                                        stream.Position = 0;

                                                        ZipArchiveEntry entry = zip.CreateEntry(BID0018_DeserializeInput.GetSuccessFileName(i));
                                                        using (var zipEntry = entry.Open())
                                                        {
                                                            stream.CopyTo(zipEntry);
                                                        }

                                                    }

                                                }

                                                var article_id = BID0018_DeserializeInput.article_id[i];
                                                RequestLog RequestLog_Insert = new RequestLog
                                                {
                                                    ContractId = ContractId,
                                                    DataSection = Data_Section.MC,
                                                    UserId = User_id,
                                                    RequestDatetime = await BatchCommonUtil.Get_Current_DateTime(),
                                                    RequestDiv = Request_Div.BID0019_DIV,
                                                    ArticleId = article_id,
                                                    SessionId = BID0018_SessionID,
                                                    ThemeId = null,
                                                    UserGroupName = UserGroupName,
                                                    ThemeGroupId = null,
                                                    TopicsId = null,
                                                    ReadCount = 1,
                                                    DemandId = BID0018_IsValidUser.First().demand_id
                                                };
                                                DbContent.RequestLog.Add(RequestLog_Insert);
                                                DbContent.SaveChanges();

                                            }
                                            else
                                            {
                                                ZipArchiveEntry entry = zip.CreateEntry(BID0018_DeserializeInput.GetErrorFileName(i));
                                                using (var zipEntry = entry.Open())
                                                {
                                                    MemoryStream stream = new MemoryStream();
                                                    stream.CopyTo(zipEntry);
                                                }
                                            }

                                        }
                                        else
                                        {                                            
                                            ZipArchiveEntry entry = zip.CreateEntry(BID0018_DeserializeInput.GetErrorFileName(i));
                                            using (var zipEntry = entry.Open())
                                            {
                                                MemoryStream stream = new MemoryStream();
                                                stream.CopyTo(zipEntry);
                                            }
                                        }

                                      
                                    }
                                    else
                                    {                                        
                                        ZipArchiveEntry entry = zip.CreateEntry(BID0018_DeserializeInput.GetErrorFileName(i));
                                        using (var zipEntry = entry.Open())
                                        {
                                            MemoryStream stream = new MemoryStream();
                                            stream.CopyTo(zipEntry);
                                        }
                                        
                                    }
                                   

                                }
                            }
                            fileBytes = zipStream.ToArray();
                        }
                        result = new HttpResponseMessage(System.Net.HttpStatusCode.OK);                        
                        result.Content = new ByteArrayContent(fileBytes);
                        result.Content.Headers.ContentType = new MediaTypeHeaderValue("application/zip");
                        result.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment")
                        {
                            FileName = "response.zip"
                        };
                    }

                }
                else
                {
                    //result = new HttpResponseMessage(System.Net.HttpStatusCode.Unauthorized);

                    result = new HttpResponseMessage(System.Net.HttpStatusCode.Unauthorized)
                    {
                        Content = new StringContent(System.Net.HttpStatusCode.Unauthorized.ToString())
                    };
                }

                return result;
            }
            catch (Exception ex)
            {
                logger.Exception(ex);
                var result = new HttpResponseMessage(System.Net.HttpStatusCode.ExpectationFailed);
                result.Content = new StringContent(ex.ToString());
                return result;
            }
            finally
            {
                logger.Trace(new MethodOutBatch(batchID));
            }

        }
    }
}
