using System.IO;
using System.Threading.Tasks;
using BatchProcess_GroupF_06.ViewModels;
using Microsoft.Azure.WebJobs;
using Microsoft.Azure.WebJobs.Extensions.Http;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using BatchProcess_GroupF_06.Helper;
using System.Collections.Generic;
using System.Linq;
using BatchProcess_GroupF_06.Models;
using System.Net.Http;
using System.IO.Compression;
using System;
using iTextSharp.text.pdf;
using CommonLibrary.Models;
using iTextSharp.text;
using BatchProcess_GroupF_06.EIF;
using System.Net.Http.Headers;
using CommonLibrary.Log;
using CommonLibrary.Log.Models.Batch;

namespace BatchProcess_GroupF_06
{
    public class BID0019
    {

        private const eBatchAPIs batchID = eBatchAPIs.BID0019;

        private ILogger_Common_Batch logger;

        public BID0019(ILogger_Common_Batch logger)
        {
            this.logger = logger;
        }
        /// <summary>
        /// This batch returns the article text (PDF) file in response to Http access from the external system.
        /// </summary>
        /// <param name="req"></param>
        /// <param name="log"></param>
        /// <returns></returns>
        [FunctionName("BID0019")]
        public  async Task<HttpResponseMessage> Run([HttpTrigger(AuthorizationLevel.Anonymous, "get", "post", Route = "v1/download/sc_articles.pdf")] HttpRequest req, ILogger log)
        {
            try
            {
                logger.Trace(new MethodInBatch(batchID));
                var result = new HttpResponseMessage(System.Net.HttpStatusCode.OK);
                byte[] fileBytes = null;
                // Reading the values from the request
                var BID0019_InputJson = req.ReadAsStringAsync().Result;
                var BID0019_DeserializeInput = JsonConvert.DeserializeObject<BID0019_ViewModel>(BID0019_InputJson);
                var validationResults = new System.Collections.Generic.List<System.ComponentModel.DataAnnotations.ValidationResult>();

                if (!Isvalid_Model.IsValid(BID0019_DeserializeInput, validationResults))
                {
                    result = new HttpResponseMessage(System.Net.HttpStatusCode.InternalServerError);
                    result.Content = new StringContent(System.Net.HttpStatusCode.InternalServerError.ToString());
                    return result;
                }
                string User_id = BID0019_DeserializeInput.api_id.ToString();
                string User_pass = BID0019_DeserializeInput.api_pass.ToString();
                string ContractCode = BID0019_DeserializeInput.contract_code.ToString();
                
                var DbContent = DB_config.db_conection();

                string hash_user_pass = BatchCommonUtil.PasswordHashing(User_pass);

                string Recive_data = JsonConvert.SerializeObject(BID0019_DeserializeInput);
                logger.Trace(new ParameterSendReceiveMethod(batchID, parameterType.Receive, Recive_data));                

                var BID0019_IsValidUser = (from c in DbContent.Contract
                                           join u in DbContent.User on c.ContractId equals u.ContractId
                                           join ug in DbContent.UserGroup on u.UserGroupId equals ug.UserGroupId
                                           where c.ContractCode == ContractCode && u.UserId == User_id && u.Password == hash_user_pass
                                           && u.DelFlag == false && c.DelFlag == false && ug.DelFlag == false
                                           select new
                                           {
                                               contract_id = u.ContractId,
                                               user_login = u.ScUserId,
                                               user_password = u.ScUserPw,
                                               user_group_name = ug.UserGroupName,
                                               demand_id = ug.DemandId
                                           }).ToList();
                if (BID0019_IsValidUser.Any())
                {
                    string BID0019_SessionID = Guid.NewGuid().ToString("D");
                    var ContractId = BID0019_IsValidUser.First().contract_id;
                    // Add record to authentication_log
                    AuthenticationLog AuthLog = new AuthenticationLog
                    {
                        ContractId = ContractId,
                        UserId = User_id,
                        SessionId = BID0019_SessionID,
                        LoginDatetime = await BatchCommonUtil.Get_Current_DateTime(),
                        FunctionSection = Function_Section.BID0019,
                        DemandId = BID0019_IsValidUser.First().demand_id,
                        UserGroupName = BID0019_IsValidUser.First().user_group_name
                    };
                    DbContent.AuthenticationLog.Add(AuthLog);
                    DbContent.SaveChanges();
                    //----------------------------------------------------------------

                    IN_CMN0F13_LoginModel ScLogin = new IN_CMN0F13_LoginModel();                    
                    foreach (var sc_login_data in BID0019_IsValidUser.Take(1))
                    {
                        ScLogin.userId = sc_login_data.user_login;
                        ScLogin.password = sc_login_data.user_password;

                        var ArticleId_Count = BID0019_DeserializeInput.article_id.Count();
                        if (ArticleId_Count > 0)
                        {
                            MemoryStream zipStream = new MemoryStream();

                            using (ZipArchive zip = new ZipArchive(zipStream, ZipArchiveMode.Create, true))
                            {
                                for (int j = 0; j < ArticleId_Count; j++)
                                {
                                    var Check_SC_article = (from sc in DbContent.ScArticle
                                                            join ta in DbContent.ThemeArticle on sc.ArticleId equals ta.ArticleId
                                                            join cn in DbContent.Contract on ta.ContractId equals cn.ContractId
                                                            where sc.ArticleId == BID0019_DeserializeInput.article_id[j].ToString()
                                                            && sc.DispStatus == 0 && cn.ContractCode == ContractCode
                                                            && sc.DelFlag == false && ta.DelFlag == false && cn.DelFlag == false && ta.DataSection == Data_Section.SC
                                                            select sc).ToList();

                                    if (Check_SC_article.Any())
                                    {
                                        string[] article = new string[1];
                                        article[0] = BID0019_DeserializeInput.article_id[j].ToString();

                                        IN_AID9027_SC_PDF_Zip BID0019_AID9026_Input = new IN_AID9027_SC_PDF_Zip
                                        {
                                            article_id = article,
                                            api_id = sc_login_data.user_login,
                                            api_pass = sc_login_data.user_password,
                                            contract_code = ContractCode
                                        };
                                        string send_data = JsonConvert.SerializeObject(BID0019_AID9026_Input);
                                        logger.Trace(new ParameterSendReceiveMethod(batchID, parameterType.Send, send_data));                                        

                                        HttpResponseMessage BID0019_Response = await AID9027.Get_AID9027_MultiplePDF_as_zip(BID0019_AID9026_Input, ScLogin);
                                        if (BID0019_Response.IsSuccessStatusCode)
                                        {
                                            MemoryStream BID0019_MS = new MemoryStream();
                                            var BID0019_Response_PDF = BID0019_Response.Content.ReadAsStreamAsync().Result;
                                            BID0019_Response_PDF.CopyTo(BID0019_MS);

                                            MemoryStream NewMemorySteam = new MemoryStream();
                                            Stream BID0019_MS_Output;
                                            ZipArchive Archive_Data = new ZipArchive(BID0019_MS);

                                            //MemoryStream zipStream = new MemoryStream();

                                           // using (ZipArchive zip = new ZipArchive(zipStream, ZipArchiveMode.Create, true))
                                           // {
                                                foreach (ZipArchiveEntry entry in Archive_Data.Entries)
                                                {
                                                    if (entry.FullName.EndsWith(".pdf", StringComparison.OrdinalIgnoreCase))
                                                    {
                                                        BID0019_MS_Output = entry.Open();
                                                        BID0019_MS_Output.CopyTo(NewMemorySteam);
                                                        byte[] bytes = NewMemorySteam.ToArray();
                                                        Font blackFont = FontFactory.GetFont("Arial", 12, Font.NORMAL, BaseColor.BLACK);
                                                        using (MemoryStream stream = new MemoryStream())
                                                        {
                                                            PdfReader reader = new PdfReader(bytes);
                                                            using (PdfStamper stamper = new PdfStamper(reader, stream))
                                                            {
                                                                stamper.Writer.CloseStream = false;
                                                                int pages = reader.NumberOfPages;
                                                                for (int i = 1; i <= pages; i++)
                                                                {
                                                                    ColumnText.ShowTextAligned(stamper.GetUnderContent(i), Element.ALIGN_RIGHT, new Phrase(i.ToString(), blackFont), 568f, 15f, 0);
                                                                }

                                                                stamper.Close();
                                                                stream.Position = 0;

                                                                ZipArchiveEntry zipentry = zip.CreateEntry(entry.FullName);
                                                                using (Stream entryStream = zipentry.Open())
                                                                {
                                                                    stream.CopyTo(entryStream);
                                                                }
                                                            }

                                                        }
                                                    }
                                                    else
                                                    {                                                        
                                                        BID0019_MS_Output = entry.Open();
                                                        ZipArchiveEntry zipentry = zip.CreateEntry(entry.FullName);
                                                        using (Stream entryStream = zipentry.Open())
                                                        {
                                                            BID0019_MS_Output.CopyTo(entryStream);
                                                        }
                                                    }
                                                }
                                           // }
                                          
                                            // Insertion on request_log table
                                            // ---------------------------------
                                            int ArticleIdLength = BID0019_DeserializeInput.article_id.Length;
                                            for (int i = 0; i < ArticleIdLength; i++)
                                            {
                                                var article_id = BID0019_DeserializeInput.article_id[i];

                                                RequestLog RequestLog_Insert = new RequestLog
                                                {
                                                    ContractId = ContractId,
                                                    DataSection = Data_Section.SC,
                                                    UserId = User_id,
                                                    RequestDatetime = await BatchCommonUtil.Get_Current_DateTime(),
                                                    RequestDiv = Request_Div.BID0019_DIV,
                                                    ArticleId = article_id,
                                                    SessionId = BID0019_SessionID,
                                                    ThemeId = null,
                                                    UserGroupName = BID0019_IsValidUser.First().user_group_name.ToString(),
                                                    ThemeGroupId = null,
                                                    TopicsId = null,
                                                    ReadCount = 1,
                                                    DemandId = BID0019_IsValidUser.First().demand_id
                                                };
                                                DbContent.RequestLog.Add(RequestLog_Insert);
                                                DbContent.SaveChanges();
                                            }
                                        }
                                        else
                                        {
                                            // returning the status code return by 9027
                                            return result = new HttpResponseMessage(BID0019_Response.StatusCode);
                                        }
                                    }
                                    else
                                    {
                                        //todo 
                                        //.err with internel server   
                                        var get_articleID = BID0019_DeserializeInput.GetErrorFileName(j);
                                        string[] split_articleID = get_articleID.Split('\\');
                                        string set_articleID = "";
                                        if(split_articleID.Length>0)
                                        {
                                            set_articleID = split_articleID[0]+".err";
                                        }
                                        else
                                        {
                                            set_articleID = get_articleID;
                                        }

                                        ZipArchiveEntry entry = zip.CreateEntry(set_articleID);

                                        using (var zipEntry = entry.Open())
                                        {
                                            MemoryStream stream = new MemoryStream();
                                            stream.CopyTo(zipEntry);
                                        }

                                    }                                    
                                }                                
                            }
                            fileBytes = zipStream.ToArray();
                        }
                    }
                    result = new HttpResponseMessage(System.Net.HttpStatusCode.OK);                                       
                    result.Content = new ByteArrayContent(fileBytes);
                    result.Content.Headers.ContentType = new MediaTypeHeaderValue("application/zip");
                    result.Content.Headers.ContentDisposition = new ContentDispositionHeaderValue("attachment")
                    {
                        FileName = "response.zip"
                    };
                }
                else
                {
                    // result = new HttpResponseMessage(System.Net.HttpStatusCode.Unauthorized);
                    result = new HttpResponseMessage(System.Net.HttpStatusCode.Unauthorized)
                    {
                        Content = new StringContent(System.Net.HttpStatusCode.Unauthorized.ToString())
                    };
                }
                return result;
            }
            catch (Exception ex)
            {
                logger.Exception(ex);
                var result = new HttpResponseMessage(System.Net.HttpStatusCode.ExpectationFailed);
                result.Content = new StringContent(ex.ToString());
                return result;
            }
            finally
            {
                logger.Trace(new MethodOutBatch(batchID));
            }


        }
    }
}
