﻿using CommonLibrary.Models;
using InternalInterface.BusinessAccessLayer.UnitOfWork;
using InternalInterface.Helpers;
using InternalInterface.ViewModels;
using iTextSharp.text;
using iTextSharp.text.pdf;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;

namespace InternalInterface.BusinessAccessLayer.Pdf_Admin
{
    public class Pdf_Admin_Service : IPdf_Admin_Service
    {
        private readonly IUnitOfWork _uow;
        public Pdf_Admin_Service(IUnitOfWork uow)
        {
            _uow = uow;
        }

        /// <summary>
        /// 設定値取得
        /// </summary>
        /// <param name="configuration_Key"></param>
        /// <returns>出力モデル</returns>
        private string get_Configuration(string configuration_Key)
        {
            // 設定値取得
            var configuration = _uow.Configuration.Get_Configuration_By_Key(configuration_Key);

            if (configuration != null)
            {
                return configuration.ConfigurationValue;
            }

            //設定値返却データ
            return string.Empty;
        }

        /// <summary>
        /// 原文取得(pdf取得)
        /// </summary>
        /// <param name="input"></param>
        /// <param name="articleInfos"></param>
        /// <param name="login_User_Id"></param>
        /// <returns>出力モデル</returns>
        public async Task<Pdf_Admin_Response_Model> get_Articles_Pdf_InfoAsync (int pdf_flag,  List<Pdf_Article_info> articleInfos, string login_User_Id)
        {
            Pdf_Admin_Response_Model pdfResponse_Models = new Pdf_Admin_Response_Model();
            //一時リスト
            List<string> mc_Article_Ids_Ok;
            //外部連携
            List<string> mc_pdf_failed_article_ids = null;
            //外部連携
            List<string> sc_pdf_failed_article_ids = null;
            //MC全件要求成功
            bool mc_Flag = true;
            //SC全件要求成功
            bool sc_Flag = true;
            //WAPI008
            WAPI008_Response_Model output_MC_pdfs = null;
            ATCD017_Response_Model output_SC_pdfs = null;
            List<AdminArticleInfoItems> articleInfoItems = new List<AdminArticleInfoItems>();

            try
            {
                //入力パラメータの記事情報配下の「新聞種別」が1の記事
                List<string> mc_Article_Ids = articleInfos.Where(s => s.data_section == 1).Select(s => s.article_id).Distinct().ToList();

                if (mc_Article_Ids.Count > 0)
                {
                    //MC失敗した記事情報
                    var mcArticleIds = _uow.Theme_Article.Get_McArticle_Info(mc_Article_Ids, ConstMessages.intMc, pdf_flag);
                    
                    AdminArticleInfoItems articleInfoItem;
                    int i = 0;
                    if (mcArticleIds != null)
                    {
                        foreach (var articleId in mcArticleIds)
                        {
                            articleInfoItem = new AdminArticleInfoItems();
                            //記事番号
                            articleInfoItem.article_no = i + 1;
                            //記事ID
                            articleInfoItem.article_id = articleId.article_id;
                            //新聞種別
                            articleInfoItem.data_section = ConstMessages.intMc;
                            //記事コメント
                            articleInfoItem.comment = articleId.comment;
                            //記事タイトル
                            articleInfoItem.title = _uow.Mc_Article.Get_Title_By_Article_Id(articleId.article_id);
                            //mc記事情報
                            articleInfoItem.mc_article_info = _uow.Mc_Article.Get_Mc_Articles_By_Article_Id(articleId.article_id).ToList();

                            articleInfoItems.Add(articleInfoItem);
                        }
                    }
                    
                    //一時リストに格納する
                    var mcArticleIds_Temporarily = _uow.Theme_Article.Get_Temporarily_McArticle_Info(mc_Article_Ids, ConstMessages.intMc, pdf_flag);

                    //一時リストにデータがある場合
                    if (mcArticleIds_Temporarily.Count() > 0)
                    {
                        //MC一時リスト
                        mc_Article_Ids_Ok = mcArticleIds_Temporarily.Select(s => s.article_id).Distinct().ToList();
                        //MCのID パスワード 取得
                        var Mc_API_User_Id = get_Configuration(ConstMessages.El_Admin_Api_Mc_Id);
                        var Mc_API_Password = get_Configuration(ConstMessages.El_Admin_Api_Mc_Pw);

                        output_MC_pdfs = await get_MC_Articles_Pdf_WAPI008_Async(Mc_API_User_Id, Mc_API_Password, login_User_Id, mc_Article_Ids_Ok);
                    }

                    //WAPI008
                    if (output_MC_pdfs != null)
                    {
                        if (output_MC_pdfs.OutputResponse != null && output_MC_pdfs.OutputResponse.Count() > 0)
                        {
                            //外部連携ファイルリスト
                            mc_pdf_failed_article_ids = output_MC_pdfs.OutputResponse.Where(s => s.Image_bytes == null).Select(s => s.Image_filename).ToList();
                        }
                    }

                    //外部連携失敗リスト
                    if (mc_pdf_failed_article_ids != null)
                    {
                        if (mc_pdf_failed_article_ids.Count() > 0)
                        {
                            mc_Flag = false;
                        }
                    }
                }

                //入力パラメータの記事情報配下の「新聞種別」が2の記事
                List<string> sc_Article_Ids = articleInfos.Where(s => s.data_section == 2).Select(s => s.article_id).Distinct().ToList();

                if (sc_Article_Ids.Count > 0)
                {
                    Dictionary<string, string> sc_ArticleIDs_Wwith_PdfKey = new Dictionary<string, string>();

                    //
                    foreach (var sc in articleInfos)
                    {
                        //入力パラメータのPDF取得キーがnull以外の記事がある場合
                        if (sc.data_section == ConstMessages.intSc && !string.IsNullOrEmpty(sc.pdf_key))
                        {
                            //一時リストに格納する
                            sc_ArticleIDs_Wwith_PdfKey.Add(sc.article_id, sc.pdf_key);
                        }
                    }

                    //SCのID パスワード 取得
                    var Sc_API_User_Id = get_Configuration(ConstMessages.El_Admin_Api_Sc_Id);
                    var Sc_API_Password = get_Configuration(ConstMessages.El_Admin_Api_Sc_Pw);

                    output_SC_pdfs = await get_SC_Articles_Pdf_ATCD017_Async(Sc_API_User_Id, Sc_API_Password, sc_ArticleIDs_Wwith_PdfKey);

                    //外部連携
                    if (output_SC_pdfs != null)
                    {
                        //外部連携有る
                        if (output_SC_pdfs.OutputResponse != null && output_SC_pdfs.OutputResponse.Count() > 0)
                        {
                            //外部連携ファイルリスト
                            sc_pdf_failed_article_ids = output_SC_pdfs.OutputResponse.Where(s => s.Image_bytes == null).Select(s => s.Image_filename).ToList();
                        }
                    }

                    //外部連携失敗リスト
                    if (sc_pdf_failed_article_ids != null)
                    {
                        if (sc_pdf_failed_article_ids.Count() > 0)
                        {
                            sc_Flag = false;
                        }
                    }
                }

                //全件要求成功の場合
                if (mc_Flag && sc_Flag)
                {
                    List<byte[]> pdfs = new List<byte[]>();
                    if (mc_pdf_failed_article_ids != null)
                    {
                        foreach (var mcpdf in output_MC_pdfs.OutputResponse)
                        {
                            byte[] pdf = mcpdf.Image_bytes; //Array.ConvertAll(mcpdf.Image_bytes, c => (uint)c);
                            pdfs.Add(pdf);
                        }
                    }
                    if (sc_pdf_failed_article_ids != null)
                    {
                        foreach (var scpdf in output_SC_pdfs.OutputResponse)
                        {
                            byte[] pdf = scpdf.Image_bytes;// Array.ConvertAll(scpdf.Image_bytes, c => (uint)c);
                            pdfs.Add(pdf);
                        }
                    }

                    //Get Merged pdfs
                    byte[] mergedpdfs = merge_pdfs(pdfs);
                    //Add page number
                    byte[] pagenumberpdf = add_pagenumber_pdfs(mergedpdfs);
                    //Set permissions
                    int pdf_permissions = PdfWriter.DO_NOT_ENCRYPT_METADATA;
                    if (pdf_flag == 2)
                    {
                        pdf_permissions = PdfWriter.DO_NOT_ENCRYPT_METADATA | PdfWriter.ALLOW_PRINTING;
                    }
                    //PDFファイルを暗号化処理する（PDFの保護に"128-bit RC4"を設定する）
                    byte[] encrypted_pdf = encrypt_pdf(pagenumberpdf, null, "ELNET123", pdf_permissions);

                    //PDFファイルまたは本文テキストを返す
                    pdfResponse_Models.data_flag = null;
                    if (encrypted_pdf != null)
                    {
                        pdfResponse_Models.pdf_file = Array.ConvertAll(encrypted_pdf, c => (uint)c);
                    }
                    pdfResponse_Models.print_flag = null;
                    pdfResponse_Models.article_text = null;
                    pdfResponse_Models.article_info = null;
                }
                else
                {
                    //一部要求失敗または全件要求失敗の場合
                    pdfResponse_Models.data_flag = 1;
                    pdfResponse_Models.print_flag = null;
                    pdfResponse_Models.pdf_file = null;
                    pdfResponse_Models.article_text = null;
                    //失敗した記事を返す
                    pdfResponse_Models.article_info = articleInfoItems;
                }

                return pdfResponse_Models;
            }
            catch (Exception e)
            {
                return null;

            }
        }

        /// <summary>
        /// 原文取得(本文テキスト取得)
        /// </summary>
        /// <param name="articleid"></param>
        /// <param name="data_section"></param>
        /// <param name="login_User_Id"></param>
        /// <returns>出力モデル</returns>
        public async Task<Pdf_Admin_Response_Model> get_Articles_Text_InfoAsync( string articleid, int data_section,int pdf_flg, string login_User_Id)
        {
            Pdf_Admin_Response_Model pdfResponse_Models = new Pdf_Admin_Response_Model();

            try
            {
                //入力パラメータの記事情報配下の「新聞種別」が1の記事
                List<string> mc_Article_Ids = new List<string>();
                mc_Article_Ids.Add(articleid);

                var mcArticleIds = _uow.Theme_Article.Get_McArticle_Info(mc_Article_Ids, ConstMessages.intMc, 3);
                List<AdminArticleInfoItems> articleInfoItems = new List<AdminArticleInfoItems>();
                AdminArticleInfoItems articleInfoItem;
                int i = 0;
                //foreach (var articleId in mcArticleIds)
                //{
                //    articleInfoItem = new AdminArticleInfoItems();
                //    //記事番号
                //    articleInfoItem.article_no = i + 1;
                //    //記事ID
                //    articleInfoItem.article_id = articleId.article_id;
                //    //新聞種別
                //    articleInfoItem.data_section = ConstMessages.intMc;
                //    //記事コメント
                //    articleInfoItem.comment = articleId.comment;
                //    //記事タイトル
                //    articleInfoItem.title = _uow.Mc_Article.Get_Title_By_Article_Id(articleId.article_id);
                //    //mc記事情報
                //    articleInfoItem.mc_article_info = _uow.Mc_Article.Get_Mc_Articles_By_Article_Id(articleId.article_id).ToList();

                //    articleInfoItems.Add(articleInfoItem);
                //}

                var Mc_API_User_Id = get_Configuration(ConstMessages.El_Admin_Api_Mc_Id);
                var Mc_API_Password = get_Configuration(ConstMessages.El_Admin_Api_Mc_Pw);
                //本文テキスト
                uint[] ArticleTextPdf = null;
                string ArticleText = null;
                if (pdf_flg == 3)
                {
                    WAPI014_Response_Model outputbodytext = await get_MC_Articles_Text_WAPI014_Async(Mc_API_User_Id, Mc_API_Password, login_User_Id, mc_Article_Ids);
                    if (outputbodytext != null && outputbodytext.OutputResponse != null && outputbodytext.OutputResponse.Count() > 0)
                    {
                        ArticleText = outputbodytext.OutputResponse.Where(s => s.Article_return_text != null).Select(s => s.Article_return_text).FirstOrDefault();
                    }
                    else
                    {
                        //Article does not have any body text
                        pdfResponse_Models.data_flag = 3;
                    }

                    if (!string.IsNullOrEmpty(ArticleText))
                    {
                        pdfResponse_Models.data_flag = null;
                        pdfResponse_Models.pdf_file = null;
                        pdfResponse_Models.print_flag = 1;
                        pdfResponse_Models.article_text = ArticleText;
                        pdfResponse_Models.article_info = null;
                    }
                    else
                    {
                        //No article exist in db
                        pdfResponse_Models.data_flag = 3;
                        pdfResponse_Models.pdf_file = null;
                        pdfResponse_Models.print_flag = null;
                        pdfResponse_Models.article_text = null;
                        pdfResponse_Models.article_info = null;
                    }
                }else if(pdf_flg == 4 )
                {
                    WAPI014_Pdf_Response_Model_User outputbodypdf = await get_MC_Articles_Text_Pdf_WAPI014_Async(Mc_API_User_Id, Mc_API_Password, login_User_Id, mc_Article_Ids);
                    if (outputbodypdf != null && outputbodypdf.OutputResponse != null && outputbodypdf.OutputResponse.Count() > 0)
                    {
                        var article_pdf_byte = outputbodypdf.OutputResponse.Where(s => s.Image_bytes != null).Select(s => s.Image_bytes).FirstOrDefault();
                        if (article_pdf_byte == null)
                        {
                            //Article does not have any body text
                            pdfResponse_Models.data_flag = 3;
                        }
                        else
                        {
                            ArticleTextPdf = Array.ConvertAll(article_pdf_byte, c => (uint)c);
                        }
                    }
                    else
                    {
                        //Article does not have any body text
                        pdfResponse_Models.data_flag = 3;
                    }
                    if (ArticleTextPdf != null)
                    {
                        pdfResponse_Models.data_flag = null;
                        pdfResponse_Models.pdf_file = ArticleTextPdf;
                        pdfResponse_Models.print_flag = 1;
                        pdfResponse_Models.article_text = null;
                        pdfResponse_Models.article_info = null;
                    }
                    else
                    {
                        //No article exist in db
                        pdfResponse_Models.data_flag = 3;
                        pdfResponse_Models.pdf_file = null;
                        pdfResponse_Models.print_flag = null;
                        pdfResponse_Models.article_text = null;
                        pdfResponse_Models.article_info = null;
                    }

                }

            }
            catch (Exception e)
            {
                pdfResponse_Models = null;
            }

            return pdfResponse_Models;
        }

        private async Task<WAPI014_Pdf_Response_Model_User> get_MC_Articles_Text_Pdf_WAPI014_Async(string API_User_Id, string API_Password, string Mc_UserID, List<string> article_ids)
        {
            WAPI014_Pdf_Response_Model_User wapi014_pdf_response = new WAPI014_Pdf_Response_Model_User();

            List<WAPI014_Pdf_Output_ViewModel_User> wapi014_pdf_Output_ViewModels = new List<WAPI014_Pdf_Output_ViewModel_User>();

            try
            {

                for (int i = 0; i < article_ids.Count(); i++)
                {
                    IN_WAPI014_ArticleText wapi014_input = new IN_WAPI014_ArticleText()
                    {
                        ac = article_ids[i],
                        dmsid = API_User_Id,
                        password = API_Password,
                        mcuserid = Mc_UserID,
                        responsetype = "pdf"
                    };
                    HttpResponseMessage textresponse = await EIF_Network_Handler.GetDataForWAPI(wapi014_input, EIF_URL_Config.MC_WAPI014_URL);
                    if (textresponse.IsSuccessStatusCode)
                    {
                        string textresponseString = await textresponse.Content.ReadAsStringAsync();
                        OP_MC_WAPI014_ArticleText oP_MC_WAPI014 = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_MC_WAPI014_ArticleText>(textresponseString);
                        if (typeof(OP_MC_WAPI014_ArticleText).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                        {
                            (oP_MC_WAPI014 as I_OP_BaseModel).SetUpCommonResponse();
                        }


                        if (oP_MC_WAPI014.ExternalAPIResonse != null && oP_MC_WAPI014.ExternalAPIResonse.IsSuccessResponse)
                        {
                            WAPI014_Pdf_Output_ViewModel_User wapi014_pdf_Output_ViewModel = new WAPI014_Pdf_Output_ViewModel_User()
                            {
                                Article_id = article_ids[i],
                                Image_bytes = Convert.FromBase64String(oP_MC_WAPI014.pdf)
                            };

                            wapi014_pdf_Output_ViewModels.Add(wapi014_pdf_Output_ViewModel);
                        }
                        else
                        {
                            WAPI014_Pdf_Output_ViewModel_User wapi014_pdf_Output_ViewModel = new WAPI014_Pdf_Output_ViewModel_User()
                            {
                                Article_id = article_ids[i],
                                Image_bytes = null
                            };

                            wapi014_pdf_Output_ViewModels.Add(wapi014_pdf_Output_ViewModel);
                        }

                    }
                    else
                    {
                        WAPI014_Pdf_Output_ViewModel_User wapi014_pdf_Output_ViewModel = new WAPI014_Pdf_Output_ViewModel_User()
                        {
                            Article_id = article_ids[i],
                            Image_bytes = null
                        };

                        wapi014_pdf_Output_ViewModels.Add(wapi014_pdf_Output_ViewModel);
                    }

                }

                wapi014_pdf_response.OutputResponse = wapi014_pdf_Output_ViewModels;
            }
            catch (Exception e)
            {
                wapi014_pdf_response.OutputResponse = null;
            }

            return wapi014_pdf_response;
        }

        private async Task<WAPI008_Response_Model> get_MC_Articles_Pdf_WAPI008_Async(string API_User_Id, string API_Password, string Mc_UserID, List<string> article_ids)
        {
            WAPI008_Response_Model bID0018_Response_Model = new WAPI008_Response_Model();

            List<WAPI008_Output_ViewModel> bID0018_Output_ViewModels = new List<WAPI008_Output_ViewModel>();

            try
            {
                for (int i = 0; i < article_ids.Count(); i++)
                {
                    IN_WAPI008_PDF in_wapi008 = new IN_WAPI008_PDF()
                    {
                        ac = article_ids[i],
                        dmsid = API_User_Id,
                        password = API_Password,
                        mcuserid = Mc_UserID
                    };
                    HttpResponseMessage pdfresponse = await EIF_Network_Handler.GetDataForWAPI(in_wapi008, EIF_URL_Config.MC_WAPI008_URL);
                    if (pdfresponse.IsSuccessStatusCode)
                    {
                        string pdfresponseString = await pdfresponse.Content.ReadAsStringAsync();
                        OP_MC_WAPI008_PDF oP_MC_WAPI008 = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_MC_WAPI008_PDF>(pdfresponseString);
                        if (typeof(OP_MC_WAPI008_PDF).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                        {
                            (oP_MC_WAPI008 as I_OP_BaseModel).SetUpCommonResponse();
                        }

                        if (oP_MC_WAPI008.ExternalAPIResonse != null && oP_MC_WAPI008.ExternalAPIResonse.IsSuccessResponse)
                        {
                            WAPI008_Output_ViewModel bID0018_Output_ViewModel = new WAPI008_Output_ViewModel()
                            {
                                Image_filename = article_ids[i],
                                Image_bytes = Convert.FromBase64String(oP_MC_WAPI008.pdf)
                            };

                            bID0018_Output_ViewModels.Add(bID0018_Output_ViewModel);
                        }
                        else
                        {
                            // if failer response return, it always return below failer response.
                            // Code review change (CRC): GT
                            WAPI008_Output_ViewModel bID0018_Output_ViewModel = new WAPI008_Output_ViewModel()
                            {
                                Image_filename = article_ids[i],
                                Image_bytes = null
                            };
                            bID0018_Output_ViewModels.Add(bID0018_Output_ViewModel);
                        }

                    }
                    else
                    {
                        WAPI008_Output_ViewModel bID0018_Output_ViewModel = new WAPI008_Output_ViewModel()
                        {
                            Image_filename = article_ids[i],
                            Image_bytes = null
                        };
                        bID0018_Output_ViewModels.Add(bID0018_Output_ViewModel);
                    }

                }
                bID0018_Response_Model.OutputResponse = bID0018_Output_ViewModels;
            }
            catch (Exception e)
            {
                bID0018_Response_Model.OutputResponse = null;
            }

            return bID0018_Response_Model;
        }

        private async Task<ATCD017_Response_Model> get_SC_Articles_Pdf_ATCD017_Async(string API_User_Id, string API_Password, Dictionary<string, string> article_ids)
        {
            ATCD017_Response_Model bID0019_Response_Model = new ATCD017_Response_Model();

            List<ATCD017_Output_ViewModel> bID0019_Output_ViewModels = new List<ATCD017_Output_ViewModel>();

            try
            {
                //Login to SC
                IN_CMN0F13_LoginModel ScLogin = new IN_CMN0F13_LoginModel()
                {
                    actionId = "CMN0F13",
                    userId = API_User_Id,
                    password = API_Password
                };
                HttpResponseMessage login_response = await EIF_Network_Handler.PostData(ScLogin, EIF_URL_Config.SC_BASE_URL);
                if (login_response.StatusCode == System.Net.HttpStatusCode.OK)
                {
                    string loginresString = await login_response.Content.ReadAsStringAsync();
                    OP_CMN0F21_LogoutModel loginResponseData = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_CMN0F21_LogoutModel>(loginresString);
                    if (typeof(OP_CMN0F21_LogoutModel).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                    {
                        (loginResponseData as I_OP_BaseModel).SetUpCommonResponse();
                    }
                    if (loginResponseData.ExternalAPIResonse != null && loginResponseData.ExternalAPIResonse.IsSuccessResponse)
                    {
                        foreach (var article in article_ids)
                        {
                            string article_id = article.Key;
                            string pdf_key = article.Value;
                            IN_ATCD017_ElectronicArticlePDFModel iN_ATCD017_ElectronicArticlePDF = new IN_ATCD017_ElectronicArticlePDFModel()
                            {
                                keyPdf = pdf_key
                            };
                            HttpResponseMessage ATCD017_res = await EIF_Network_Handler.PostDataWithCookieLogin(iN_ATCD017_ElectronicArticlePDF, EIF_URL_Config.SC_BASE_URL, login_response);
                            if (ATCD017_res.StatusCode == System.Net.HttpStatusCode.OK)
                            {
                                MemoryStream MS_pdf = new MemoryStream();
                                var Result_pdf = ATCD017_res.Content.ReadAsStreamAsync().Result;

                                Result_pdf.CopyTo(MS_pdf);
                                MS_pdf.Position = 0;

                                ATCD017_Output_ViewModel bID0019_Output_ViewModel = new ATCD017_Output_ViewModel()
                                {
                                    Image_filename = article_id,
                                    Image_bytes = MS_pdf.ToArray()
                                };

                                bID0019_Output_ViewModels.Add(bID0019_Output_ViewModel);
                            }
                            else
                            {
                                ATCD017_Output_ViewModel bID0019_Output_ViewModel = new ATCD017_Output_ViewModel()
                                {
                                    Image_filename = article_id,
                                    Image_bytes = null
                                };

                                bID0019_Output_ViewModels.Add(bID0019_Output_ViewModel);
                            }
                        }

                        //Calling Logout api
                        IN_CMN0F21_LogoutModel _LogoutModel = new IN_CMN0F21_LogoutModel()
                        {
                            actionId = "CMN0F21"
                        };
                        HttpResponseMessage _logoutresponse = await EIF_Network_Handler.PostDataWithCookieLogin(_LogoutModel, EIF_URL_Config.SC_BASE_URL, login_response);

                        bID0019_Response_Model.OutputResponse = bID0019_Output_ViewModels;
                    }
                    else
                    {
                        bID0019_Response_Model.OutputResponse = null;
                    }
                }
            }
            catch (Exception e)
            {
                bID0019_Response_Model.OutputResponse = null;
            }

            return bID0019_Response_Model;
        }

        private async Task<WAPI014_Response_Model> get_MC_Articles_Text_WAPI014_Async(string API_User_Id, string API_Password, string Mc_UserID, List<string> article_ids)
        {
            WAPI014_Response_Model bID0022_Response_Model = new WAPI014_Response_Model();

            List<WAPI014_Output_ViewModel> bID0022_Output_ViewModels = new List<WAPI014_Output_ViewModel>();

            try
            {
                for (int i = 0; i < article_ids.Count(); i++)
                {
                    IN_WAPI014_ArticleText wapi014_input = new IN_WAPI014_ArticleText()
                    {
                        ac = article_ids[i],
                        dmsid = API_User_Id,
                        password = API_Password,
                        mcuserid = Mc_UserID,
                        responsetype = "text"
                    };

                    HttpResponseMessage textresponse = await EIF_Network_Handler.GetDataForWAPI(wapi014_input, EIF_URL_Config.MC_WAPI014_URL);
                    if (textresponse.IsSuccessStatusCode)
                    {
                        string textresponseString = await textresponse.Content.ReadAsStringAsync();
                        OP_MC_WAPI014_ArticleText oP_MC_WAPI014 = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_MC_WAPI014_ArticleText>(textresponseString);
                        if (typeof(OP_MC_WAPI014_ArticleText).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                        {
                            (oP_MC_WAPI014 as I_OP_BaseModel).SetUpCommonResponse();
                        }

                        if (oP_MC_WAPI014.ExternalAPIResonse != null && oP_MC_WAPI014.ExternalAPIResonse.IsSuccessResponse)
                        {
                            WAPI014_Output_ViewModel bID0022_Output_ViewModel = new WAPI014_Output_ViewModel()
                            {
                                Article_id = article_ids[i],
                                Article_return_text = oP_MC_WAPI014.text
                            };

                            bID0022_Output_ViewModels.Add(bID0022_Output_ViewModel);
                        }
                        else
                        {
                            WAPI014_Output_ViewModel bID0022_Output_ViewModel = new WAPI014_Output_ViewModel()
                            {
                                Article_id = article_ids[i],
                                Article_return_text = null
                            };

                            bID0022_Output_ViewModels.Add(bID0022_Output_ViewModel);
                        }

                    }
                    else
                    {
                        WAPI014_Output_ViewModel bID0022_Output_ViewModel = new WAPI014_Output_ViewModel()
                        {
                            Article_id = article_ids[i],
                            Article_return_text = null
                        };

                        bID0022_Output_ViewModels.Add(bID0022_Output_ViewModel);
                    }

                }

                bID0022_Response_Model.OutputResponse = bID0022_Output_ViewModels;
            }
            catch (Exception e)
            {
                bID0022_Response_Model.OutputResponse = null;
            }

            return bID0022_Response_Model;
        }
        private byte[] merge_pdfs(List<byte[]> byte_pdfs)
        {
            byte[] bytes_merge = null;
            try
            {
                using (MemoryStream ms = new MemoryStream())
                {
                    //Using the ItextSharp Document
                    using (Document document = new Document(PageSize.A4))
                    {
                        //Using the ITextSharp PdfCopy to create a PDF document in the memory stream
                        using (PdfCopy copy = new PdfCopy(document, ms))
                        {
                            //Open the document before any changes can be made.
                            document.Open();

                            //Loop through each file
                            foreach (var pdf in byte_pdfs)
                            {
                                // Convert the BinaryStream of the file to a ITexSharp PdfReader
                                PdfReader reader = new PdfReader(pdf);
                                int n = reader.NumberOfPages;
                                //Loop through each page in the current PDF file
                                for (int page = 0; page < n;)
                                {
                                    //Import the page to the PDF document in the memory stream.
                                    copy.AddPage(copy.GetImportedPage(reader, ++page));
                                }
                            }
                        }

                        //Convert the final memory stream into a byte array.
                        bytes_merge = ms.ToArray();
                        var merge_base64 = Convert.ToBase64String(bytes_merge);
                        document.Close();
                    }

                }
            }
            catch (Exception e)
            {

            }
            return bytes_merge;
        }

        private byte[] add_pagenumber_pdfs(byte[] byte_pdf)
        {
            byte[] bytes_pagenumber_pdf = null;
            try
            {
                Font blackFont = FontFactory.GetFont("Arial", 12, Font.NORMAL, BaseColor.BLACK);
                using (MemoryStream stream = new MemoryStream())
                {
                    PdfReader reader = new PdfReader(byte_pdf);
                    using (PdfStamper stamper = new PdfStamper(reader, stream))
                    {
                        //stamper.Writer.CloseStream = false;
                        int pages = reader.NumberOfPages;
                        for (int i = 1; i <= pages; i++)
                        {
                            ColumnText.ShowTextAligned(stamper.GetUnderContent(i), Element.ALIGN_RIGHT, new Phrase(i.ToString(), blackFont), 568f, 15f, 0);
                        }

                        //stamper.Close();
                    }

                    bytes_pagenumber_pdf = stream.ToArray();

                }
                string test_base64 = Convert.ToBase64String(bytes_pagenumber_pdf);

            }
            catch (Exception e)
            {

            }
            return bytes_pagenumber_pdf;
        }

        private byte[] encrypt_pdf(byte[] byte_pdf, string user_password, string admin_password, int permissions)
        {
            byte[] bytes_encrypt_pdf = null;
            try
            {

                using (MemoryStream stream = new MemoryStream())
                {
                    PdfReader reader = new PdfReader(byte_pdf);
                    using (PdfStamper stamper = new PdfStamper(reader, stream))
                    {
                        System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
                        var up = user_password != null ? Encoding.GetBytes(user_password) : null;
                        stamper.SetEncryption(up, Encoding.GetBytes(admin_password), permissions, PdfWriter.STRENGTH128BITS);
                        stamper.Close();
                    }
                    bytes_encrypt_pdf = stream.ToArray();

                }
                string test_base64 = Convert.ToBase64String(bytes_encrypt_pdf);

            }
            catch (Exception e)
            {

            }
            return bytes_encrypt_pdf;
        }
    }
}

