﻿using CommonLibrary.Models;
using InternalInterface.BusinessAccessLayer.UnitOfWork;
using InternalInterface.Helpers;
using InternalInterface.Models;
using InternalInterface.ViewModels;
using iTextSharp.text;
using iTextSharp.text.pdf;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace InternalInterface.BusinessAccessLayer.Pdf_Service
{
    public class Pdf_Service : IPdf_Service
    {
        private readonly IUnitOfWork _uow;
        public Pdf_Service(IUnitOfWork uow)
        {
            _uow = uow;
        }

        public bool isCollective_Demand_Max_Count_Exceeded(int contract_id, List<Pdf_Article_info_User> articleInfos)
        {
            try
            {
                var article_count = articleInfos.Count();
                var collective_demand_max_count = _uow.Company_Configuration.Find(s => s.ContractId == contract_id && s.ConfigurationKey == "COLLECTIVE_DEMAND_MAX_COUNT")
                                                                            .Select(s => s.ConfigurationValue).FirstOrDefault();
                if (collective_demand_max_count != null)
                {
                    if (article_count > Int32.Parse(collective_demand_max_count))
                    {
                        return true;
                    }
                }
                return false;
            }
            catch (Exception e)
            {
                return false;
            }
        }

        public async Task<PdfResponse_Model> get_Articles_Pdf_InfoAsync(int contract_id,int pdf_flag, string user_id, List<Pdf_Article_info_User> articleInfos, string Screen_ID, bool theme_browsing,bool isMail)
        {
            PdfResponse_Model pdfResponse_Models = new PdfResponse_Model();
            //Create an time zone.
            TimeZoneInfo timeZoneInfo = TimeZoneInfo.FindSystemTimeZoneById("Tokyo Standard Time");
            //Get current utc time 
            DateTimeOffset dateTimeOffset = DateTimeOffset.UtcNow;
            //Convert the current time to japan time.
            DateTimeOffset currentime = dateTimeOffset.ToOffset(timeZoneInfo.BaseUtcOffset);

            try
            {


                List<string> mc_article_ids = articleInfos.Where(s => s.Data_section == 1).Select(s => s.Article_id).Distinct().ToList();
                List<string> sc_article_ids = articleInfos.Where(s => s.Data_section == 2).Select(s => s.Article_id).Distinct().ToList();

                //Key: pk, value: article id
                Dictionary<string, string> mc_article_mappings = new Dictionary<string, string>();
                Dictionary<string, string> sc_article_mappings = new Dictionary<string, string>();
                //If ismail flag true, convert the table primary key article ids to article ids
                if (isMail)
                {
                    var tmp_mc_article_mappings = _uow.Mc_Article.Find(s => mc_article_ids.Any(mc_id => mc_id == s.McArticleId) && s.DelFlag == false)
                                                .ToLookup(kp => kp.McArticleId.Trim());
                    
                    mc_article_mappings = tmp_mc_article_mappings.ToDictionary(g => g.Key, g => g.First().ArticleId);
                    mc_article_ids = mc_article_mappings.Select(s => s.Value).Distinct().ToList();

                    var tmp_sc_article_mappings = _uow.Sc_Article.Find(s => sc_article_ids.Any(mc_id => mc_id == s.ScArticleId) && s.DelFlag == false)
                                                .ToLookup(kp => kp.ScArticleId.Trim());
                    sc_article_mappings = tmp_sc_article_mappings.ToDictionary(g => g.Key, g => g.First().ArticleId);
                    sc_article_ids = sc_article_mappings.Select(s => s.Value).Distinct().ToList();
                   
                }
               
                

                var user_group_info = _uow.User_Group.Get_UserGroup_Info_Of_User(contract_id, user_id).ToList();
                int user_group_id = user_group_info.Select(s => s.User_group_id).FirstOrDefault();
                //Get MC and SC upper limit
                int mc_upper_limit = user_group_info.Where(s => s.Data_section == 1).Select(s => s.Use_max_count).FirstOrDefault();
                int sc_upper_limit = user_group_info.Where(s => s.Data_section == 2).Select(s => s.Use_max_count).FirstOrDefault();
            

                //Get MC and SC used count
                var SC_MC_used = _uow.Charge_Total.Find(s => s.ContractId == contract_id
                                                            && (s.DataSection == 1 || s.DataSection == 2)
                                                            && s.UserId == user_id
                                                            && s.ProcessDate.Month == currentime.Month)
                                                   .ToList();
                int MC_UseCount = SC_MC_used.Where(s => s.DataSection == 1).Sum(s => s.PdfCount + s.PrintCount + s.TextDispCount + s.TextPrintCount);
                int SC_UseCount = SC_MC_used.Where(s => s.DataSection == 2).Sum(s => s.PdfCount + s.PrintCount);

                //Check MC limit exceeded
                bool mc_limit_exceeded = MC_UseCount > mc_upper_limit;
                bool sc_limit_exceeded = SC_UseCount > sc_upper_limit;

                //If user limit exceeded
                if (mc_limit_exceeded || sc_limit_exceeded)
                {
                    //Set Data flag to 0 and process ends
                    pdfResponse_Models.Data_flag = 0;
                    pdfResponse_Models.Pdf_file = null;
                    pdfResponse_Models.Print_flag = null;
                    pdfResponse_Models.Article_text = null;
                    pdfResponse_Models.Article_info = null;
                }
                else
                {
                    List<ArticleInfoItems> articleInfoItems = new List<ArticleInfoItems>();
                    var user_info = _uow.User.Get_All_User_And_UserService_Linkages(s => s.UserId == user_id 
                                                                                        && s.UserGroupId == user_group_id
                                                                                        && s.ContractId == contract_id 
                                                                                        && s.DelFlag == false).ToList();
                
                    var contract_info = _uow.Contract.Get_Contract_Info(s => s.ContractId == contract_id 
                                                                                && s.DelFlag == false).ToList();

                    //MC API Credentials
                    string Mc_API_User_Id = contract_info.Where(s => s.Data_section == 1).Select(s => s.Api_id).FirstOrDefault();
                    string Mc_API_Password = contract_info.Where(s => s.Data_section == 1).Select(s => s.Api_Password).FirstOrDefault();
                    //string Mc_UserID = user_group_info.Select(s => s.Demand_id).FirstOrDefault();
                    string Mc_UserID = user_id;

                    //SC API Credentials
                    string SC_API_User_Id = user_info.Select(s => s.Sc_user_id).FirstOrDefault();
                    string Sc_API_Password = user_info.Select(s => s.Sc_user_password).FirstOrDefault();

                    int mc_user_authority = user_info.Where(s => s.Data_section == 1).Select(s => s.Usl_authority).FirstOrDefault();
                    int sc_user_authority = user_info.Where(s => s.Data_section == 2).Select(s => s.Usl_authority).FirstOrDefault();

                    //Check user permissions
                    bool mc_pdf_CanView = mc_user_authority == 2 || mc_user_authority == 3;
                    bool mc_pdf_CanPrint = mc_user_authority == 3;
                    bool sc_pdf_CanView = sc_user_authority == 3 || sc_user_authority == 5;
                    bool sc_pdf_CanPrint = sc_user_authority == 5;

                    //Get theme article information
                    var themeArticles = _uow.Theme_Article.Find(s => s.ContractId == contract_id &&
                                                                    (mc_article_ids.Any(mc_id => mc_id == s.ArticleId) || sc_article_ids.Any(sc_id => sc_id == s.ArticleId)) &&
                                                                    s.DelFlag == false).ToList();

                    //Get MC Article Infos
                    List<McArticleInfoWithComments> mc_article_infos = _uow.Mc_Article.get_mc_article_info_with_comments(s => mc_article_ids.Any(mc_id => mc_id == s.ArticleId),contract_id).ToList();
                    //Get SC Article Infos
                    List<ScArticleInfoWithComments> sc_article_infos = _uow.Sc_Article.get_sc_article_info_with_comments(s => sc_article_ids.Any(sc_id => sc_id == s.ArticleId), contract_id).ToList();

                    //Get Display state of articles
                    var article_display_states = _uow.Article_Display_State.Find(s => s.ContractId == contract_id &&
                                                                                    (mc_article_ids.Any(mc_id => mc_id == s.ArticleId) ||
                                                                                     sc_article_ids.Any(sc_id => sc_id == s.ArticleId)) &&
                                                                                    s.DelFlag == false).ToList();

                    //Get MC Display state of articles
                    var mc_article_display_states = article_display_states.Where(s=>s.DataSection == 1).ToList();

                    //Get SC Display state of articles
                    var sc_article_display_states = article_display_states.Where(s => s.DataSection == 2).ToList();

                    //Mc Article Ids which satisfies all the conditions
                    List<string> valid_MC_ArticleIDs = new List<string>();
                    //Mc Article Ids which does not satisfy any one of the condition
                    List<string> invalid_MC_ArticleIDs = new List<string>();
                    //Mc articles which does not exist in table
                    List<string> non_Exist_MC_ArticleIDs = new List<string>();

                    //Sc Article Ids which satisfies all the conditions
                    List<string> valid_SC_ArticleIDs = new List<string>();
                    //Sc Article Ids which does not satisfy any one of the condition
                    List<string> invalid_SC_ArticleIDs = new List<string>();
                    //Sc articles which does not exist in table
                    List<string> non_Exist_SC_ArticleIDs = new List<string>();

                    //Sc article ids with pdf key
                    Dictionary<string, string> SC_ArticleIDs_with_pdfKey = new Dictionary<string, string>();

                    //Get the list of theme ids based on theme browsing
                    List<string> theme_browsing_theme_ids = getThemeBrowsing_ThemeIDs(theme_browsing,user_group_id,contract_id);

                              
                    //For MC Articles filter valid and in valid articles
                    foreach(string article_id in mc_article_ids)
                    {
                        McArticleInfoWithComments article_info = mc_article_infos.Where(s => s.Article_id == article_id).FirstOrDefault();
                        if (article_info != null)
                        {
                            bool isOriginalPdf_Not_Available = article_info.No_image_flag;
                            bool isArticle_Not_Displayable = article_info.Disp_status == 3;
                            bool isArticle_Not_DisplayableFor_User = false;
                        
                            if (!Screen_ID.Equals("SID1024"))
                            {
                                isArticle_Not_DisplayableFor_User = mc_article_display_states.Where(s => s.ContractId == contract_id
                                                                                                    && s.ArticleId == article_id
                                                                                                    && s.DispStatus == 3).Count() > 0;
                            }


                            if (isOriginalPdf_Not_Available || isArticle_Not_Displayable || isArticle_Not_DisplayableFor_User)
                            {
                                invalid_MC_ArticleIDs.Add(article_id);
                            }
                            else
                            {
                                List<string> current_Article_theme_ids = themeArticles.Where(s => s.ArticleId == article_id && s.DataSection == 1).Select(s => s.ThemeId).Distinct().ToList();
                                if (Screen_ID.StartsWith("SID0") && !theme_browsing_theme_ids.Intersect(current_Article_theme_ids).Any())
                                {
                                    invalid_MC_ArticleIDs.Add(article_id);
                                }
                                else
                                {
                                    //pdf display
                                    if (pdf_flag == 1 && !mc_pdf_CanView)
                                    {
                                        invalid_MC_ArticleIDs.Add(article_id);

                                    }
                                    //pdf print
                                    else if(pdf_flag == 2 && !mc_pdf_CanPrint)
                                    {
                                        invalid_MC_ArticleIDs.Add(article_id);
                                    }
                                    else
                                    {
                                        valid_MC_ArticleIDs.Add(article_id);
                                    }
                                
                                }
                            }
                        
                        }
                        else
                        {
                            non_Exist_MC_ArticleIDs.Add(article_id);
                        }   

                    }

                    //For SC Articles
                    //For SC Articles filter valid and in valid articles
                    foreach (string article_id in sc_article_ids)
                    {
                        ScArticleInfoWithComments article_info = sc_article_infos.Where(s => s.Article_id == article_id).FirstOrDefault();
                        if (article_info != null)
                        {
                            bool isOriginalPdf_Not_Available = string.IsNullOrEmpty(article_info.Pdf_key);
                            bool isArticle_Not_Displayable = article_info.Disp_status == 3;
                            bool isArticle_Not_DisplayableFor_User = false;

                            if (!Screen_ID.Equals("SID1024"))
                            {
                                isArticle_Not_DisplayableFor_User = sc_article_display_states.Where(s => s.ContractId == contract_id
                                                                                                    && s.ArticleId == article_id
                                                                                                    && s.DispStatus == 3).Count() > 0;
                            }


                            if (isOriginalPdf_Not_Available || isArticle_Not_Displayable || isArticle_Not_DisplayableFor_User)
                            {
                                invalid_SC_ArticleIDs.Add(article_id);
                            }
                            else
                            {
                                List<string> current_Article_theme_ids = themeArticles.Where(s => s.ArticleId == article_id && s.DataSection == 2).Select(s => s.ThemeId).Distinct().ToList();
                                if (Screen_ID.StartsWith("SID0") && !theme_browsing_theme_ids.Intersect(current_Article_theme_ids).Any())
                                {
                                    invalid_SC_ArticleIDs.Add(article_id);
                                }
                                else
                                {
                                    //pdf display
                                    if (pdf_flag == 1 && !sc_pdf_CanView)
                                    {
                                        invalid_SC_ArticleIDs.Add(article_id);

                                    }
                                    //pdf print
                                    else if (pdf_flag == 2 && !sc_pdf_CanPrint)
                                    {
                                        invalid_SC_ArticleIDs.Add(article_id);
                                    }
                                    else
                                    {
                                        valid_SC_ArticleIDs.Add(article_id);
                                        SC_ArticleIDs_with_pdfKey.Add(article_id, article_info.Pdf_key);
                                    }

                                }
                            }

                        }
                        else
                        {
                            non_Exist_SC_ArticleIDs.Add(article_id);
                        }

                    }


                    //Get MC Pdf's
                    WAPI008_Response_Model_User output_MC_pdfs = null;
                    //Get SC Pdf's
                    ATCD017_Response_Model_User output_SC_pdfs = null;
                    //Mc article ids which are failed to get pdf from the external system
                    List<string> mc_pdf_failed_article_ids = null;
                    //Sc article ids which are failed to get pdf from the external system
                    List<string> sc_pdf_failed_article_ids = null;

                    bool is_MC_Article_Empty = mc_article_ids.Count() == 0;
                    bool is_SC_Article_Empty = sc_article_ids.Count() == 0;

                    
                    //If all article ids are valid
                    if (valid_MC_ArticleIDs.Count()>0 && invalid_MC_ArticleIDs.Count() == 0)
                    {
                        output_MC_pdfs = await get_MC_Articles_Pdf_WAPI008_Async(Mc_API_User_Id, Mc_API_Password, Mc_UserID, valid_MC_ArticleIDs);
                        if (output_MC_pdfs != null)
                        {
                            if (output_MC_pdfs.OutputResponse != null && output_MC_pdfs.OutputResponse.Count() > 0)
                            {
                                mc_pdf_failed_article_ids = output_MC_pdfs.OutputResponse.Where(s => s.Image_bytes == null).Select(s => s.Image_filename).ToList();
                                if (mc_pdf_failed_article_ids != null && mc_pdf_failed_article_ids.Count() > 0)
                                {
                                    invalid_MC_ArticleIDs.AddRange(mc_pdf_failed_article_ids);
                                }
                            }
                            else
                            {
                                invalid_MC_ArticleIDs.AddRange(valid_MC_ArticleIDs);
                            }


                        }
                        else
                        {
                            invalid_MC_ArticleIDs.AddRange(valid_MC_ArticleIDs);
                        }
                    }
                    if (valid_SC_ArticleIDs.Count() > 0 && invalid_SC_ArticleIDs.Count() == 0)
                    {
                        
                        output_SC_pdfs = await get_SC_Articles_Pdf_ATCD017_Async(SC_API_User_Id, Sc_API_Password, SC_ArticleIDs_with_pdfKey);
                        if (output_SC_pdfs != null)
                        {
                            if (output_SC_pdfs.OutputResponse != null && output_SC_pdfs.OutputResponse.Count() > 0)
                            {
                                sc_pdf_failed_article_ids = output_SC_pdfs.OutputResponse.Where(s => s.Image_bytes == null).Select(s => s.Image_filename).ToList();
                                if (sc_pdf_failed_article_ids != null && sc_pdf_failed_article_ids.Count() > 0)
                                {
                                    invalid_SC_ArticleIDs.AddRange(sc_pdf_failed_article_ids);
                                }
                            }
                            else
                            {
                                invalid_SC_ArticleIDs.AddRange(valid_SC_ArticleIDs);
                            }

                        }
                        else
                        {
                            invalid_SC_ArticleIDs.AddRange(valid_SC_ArticleIDs);
                        }
                    }



                    List<string> all_invalid_article_ids = new List<string>();
                    
                    all_invalid_article_ids.AddRange(invalid_MC_ArticleIDs);
                    all_invalid_article_ids.AddRange(invalid_SC_ArticleIDs);
                   
                    //If All requests are success full
                    if (all_invalid_article_ids.Count() == 0)
                    {
                        List<byte[]> pdfs = new List<byte[]>();
                        //if (mc_pdf_failed_article_ids != null)
                        if (mc_pdf_failed_article_ids != null)
                        {
                            foreach (var mcpdf in output_MC_pdfs.OutputResponse)
                            {
                                byte[] pdf = mcpdf.Image_bytes; //Array.ConvertAll(mcpdf.Image_bytes, c => (uint)c);
                                pdfs.Add(pdf);
                            }
                        }
                        //if (sc_pdf_failed_article_ids != null)
                        if (sc_pdf_failed_article_ids != null)
                        {
                            foreach (var scpdf in output_SC_pdfs.OutputResponse)
                            {
                                byte[] pdf = scpdf.Image_bytes;// Array.ConvertAll(scpdf.Image_bytes, c => (uint)c);
                                pdfs.Add(pdf);
                            }
                        }

                        //Get Merged pdfs
                        byte[] mergedpdfs = merge_pdfs(pdfs);
                        //Add page number
                        byte[] pagenumberpdf = add_pagenumber_pdfs(mergedpdfs);
                        //Set permissions
                        int pdf_permissions = PdfWriter.DO_NOT_ENCRYPT_METADATA;
                        if (pdf_flag == 2)
                        {
                            pdf_permissions = PdfWriter.DO_NOT_ENCRYPT_METADATA | PdfWriter.ALLOW_PRINTING;
                        }
                        //Encrypt pdf by 128 bit RC4
                        byte[] encrypted_pdf = encrypt_pdf(pagenumberpdf, null, "ELNET123", pdf_permissions);

                        pdfResponse_Models.Data_flag = null;
                        pdfResponse_Models.Pdf_file = Array.ConvertAll(encrypted_pdf, c => (uint)c);
                        pdfResponse_Models.Print_flag = null;
                        pdfResponse_Models.Article_text = null;
                        pdfResponse_Models.Article_info = null;
                    }
                    else
                    {
                        int article_no = 1;
                        foreach(var article_info in articleInfos)
                        {
                            string article_id = article_info.Article_id;
                            
                            int data_section = article_info.Data_section.Value;
                            if (isMail)
                            {
                                switch (data_section)
                                {
                                    case 1:
                                        string converted_mc_article_id;
                                        if (mc_article_mappings.TryGetValue(article_id, out converted_mc_article_id))
                                        {
                                            article_id = converted_mc_article_id;
                                        }                                    
                                        break;
                                    case 2:
                                        string converted_sc_article_id;
                                        if (sc_article_mappings.TryGetValue(article_id, out converted_sc_article_id))
                                        {
                                            article_id = converted_sc_article_id;
                                        }
                                        break;
                                }
                                
                            }
                            bool isArticle_Invalid = invalid_MC_ArticleIDs.Contains(article_id) || invalid_SC_ArticleIDs.Contains(article_id);
                            if (!non_Exist_MC_ArticleIDs.Contains(article_id) && !non_Exist_SC_ArticleIDs.Contains(article_id) && isArticle_Invalid)
                            {
   
                                ArticleInfoItems articleInfoItem = new ArticleInfoItems();
                                if (isMail)
                                {
                                    articleInfoItem.Article_id = article_info.Article_id;
                                }
                                else
                                {
                                    articleInfoItem.Article_id = article_id;
                                }
                                
                                articleInfoItem.Article_no = article_no;
                                articleInfoItem.Data_section = data_section;
                                if (data_section == 1)
                                {
                                    var mc_article = mc_article_infos.Where(s => s.Article_id == article_id).FirstOrDefault();
                                    articleInfoItem.Comment = mc_article.Comment;
                                    articleInfoItem.Title = mc_article.Title;
                                    articleInfoItem.Mc_article_info = new List<McArticleInfos>()
                                    {
                                        new McArticleInfos()
                                        {
                                            Media_name = mc_article.Media_name,
                                            Issue_date = mc_article.Issue_date,
                                            Issue_type = mc_article.Issue_type,
                                            Frame_no = mc_article.Frame_no,
                                            Head_no = mc_article.Head_no,
                                            Trace = mc_article.Trace,
                                            Image_count = mc_article.Image_count,
                                            Note = mc_article.Note,
                                            Search_word = themeArticles.Where(s=>s.ArticleId == mc_article.Article_id &&
                                                                            s.DataSection == 1 &&
                                                                            !string.IsNullOrEmpty(s.SearchWord))
                                                                            .Select(s=>s.SearchWord).Distinct().ToArray()
                                        }
                                    };
                                
                                    articleInfoItem.Sc_article_info = null;
                                }
                                else
                                {
                                    var sc_article = sc_article_infos.Where(s => s.Article_id == article_id).FirstOrDefault();
                                    articleInfoItem.Title = sc_article.Title;
                                    articleInfoItem.Comment = sc_article.Comment;
                                    articleInfoItem.Sc_article_info = new List<ScArticleInfos>()
                                    {
                                        new ScArticleInfos()
                                        {
                                            Media_name = sc_article.Media_name,
                                            Issue_date = sc_article.Issue_date,
                                            Issue_type = sc_article.Issue_type,
                                            Frame_no = sc_article.Frame_no,
                                            Char_count = sc_article.Char_count,
                                            Search_word = themeArticles.Where(s=>s.ArticleId == sc_article.Article_id &&
                                                                            s.DataSection == 2 &&
                                                                            !string.IsNullOrEmpty(s.SearchWord))
                                                                            .Select(s=>s.SearchWord).Distinct().ToArray()

                                        }
                                    };
                                    articleInfoItem.Mc_article_info = null;
                                }
                                articleInfoItems.Add(articleInfoItem);
                                article_no += 1;
                            }
                        
                           
                        }
                        pdfResponse_Models.Data_flag = 1;
                        pdfResponse_Models.Print_flag = null;
                        pdfResponse_Models.Pdf_file = null;
                        pdfResponse_Models.Article_text = null;
                        pdfResponse_Models.Article_info = articleInfoItems;

                    }

                }
            }
            catch (Exception e)
            {
                pdfResponse_Models = null;
            }
            return pdfResponse_Models;
        }

        public async Task<PdfResponse_Model> get_Articles_Text_InfoAsync(int contract_id,int pdf_flag,string user_id, string articleid,int data_section, string Screen_ID, bool theme_browsing, bool isMail)
        {
            PdfResponse_Model pdfResponse_Models = new PdfResponse_Model();
            //Create an time zone.
            TimeZoneInfo timeZoneInfo = TimeZoneInfo.FindSystemTimeZoneById("Tokyo Standard Time");
            //Get current utc time 
            DateTimeOffset dateTimeOffset = DateTimeOffset.UtcNow;
            //Convert the current time to japan time.
            DateTimeOffset currentime = dateTimeOffset.ToOffset(timeZoneInfo.BaseUtcOffset);
            try
            {
                //If ismail flag true, convert the table primary key article ids to article ids
                if (isMail)
                {
                    if (data_section == 1)
                    {
                        articleid = _uow.Mc_Article.Find(s => s.McArticleId == articleid && s.DelFlag == false)
                                                .Select(s => s.ArticleId).FirstOrDefault();

                    }
                    else
                    {
                        articleid = _uow.Sc_Article.Find(s => s.ScArticleId == articleid && s.DelFlag == false)
                                    .Select(s => s.ArticleId).FirstOrDefault();
                    }
                    
                    
                }
                var user_group_info = _uow.User_Group.Get_UserGroup_Info_Of_User(contract_id, user_id).ToList();
                int user_group_id = user_group_info.Select(s => s.User_group_id).FirstOrDefault();
                //Get MC and SC upper limit
                int mc_upper_limit = user_group_info.Where(s => s.Data_section == 1).Select(s => s.Use_max_count).FirstOrDefault();
                int sc_upper_limit = user_group_info.Where(s => s.Data_section == 2).Select(s => s.Use_max_count).FirstOrDefault();


                //Get MC and SC used count
                var SC_MC_used = _uow.Charge_Total.Find(s => s.ContractId == contract_id
                                                            && (s.DataSection == 1 || s.DataSection == 2)
                                                            && s.UserId == user_id
                                                            && s.ProcessDate.Month == currentime.Month)
                                                   .ToList();
                int MC_UseCount = SC_MC_used.Where(s => s.DataSection == 1).Sum(s => s.PdfCount + s.PrintCount + s.TextDispCount + s.TextPrintCount);
                int SC_UseCount = SC_MC_used.Where(s => s.DataSection == 2).Sum(s => s.PdfCount + s.PrintCount);

                //Check MC limit exceeded
                bool mc_limit_exceeded = MC_UseCount > mc_upper_limit;
                bool sc_limit_exceeded = SC_UseCount > sc_upper_limit;

                if (mc_limit_exceeded || sc_limit_exceeded)
                {
                    //Set Data flag to 0 and process ends
                    pdfResponse_Models.Data_flag = 0;
                }
                else
                {
                    var user_info = _uow.User.Get_All_User_And_UserService_Linkages(s => s.UserId == user_id && s.UserGroupId == user_group_id && s.ContractId == contract_id && s.DelFlag == false).ToList();
                    int mc_user_authority = user_info.Where(s => s.Data_section == 1).Select(s => s.Usl_authority).FirstOrDefault();
                    int sc_user_authority = user_info.Where(s => s.Data_section == 2).Select(s => s.Usl_authority).FirstOrDefault();


                    var contract_info = _uow.Contract.Get_Contract_Info(s => s.ContractId == contract_id && s.DelFlag == false).ToList();

                    //MC API Credentials
                    string Mc_API_User_Id = contract_info.Where(s => s.Data_section == 1).Select(s => s.Api_id).FirstOrDefault();
                    string Mc_API_Password = contract_info.Where(s => s.Data_section == 1).Select(s => s.Api_Password).FirstOrDefault();
                    string Mc_UserID = user_id;


                    bool mc_text_CanView = mc_user_authority == 2 || mc_user_authority == 3;
                    bool mc_text_CanPrint = mc_user_authority == 3;
                    //bool sc_text_CanView = sc_user_authority == 3 || sc_user_authority == 5;
                    //bool sc_text_CanPrint = sc_user_authority == 5;
                    //Get search words
                    var themeArticles = _uow.Theme_Article.Find(s => s.ContractId == contract_id
                                                                    && s.ArticleId == articleid
                                                                    && s.DelFlag == false).ToList();

                    List<string> theme_browsing_theme_ids = getThemeBrowsing_ThemeIDs(theme_browsing, user_group_id, contract_id);
                    if (data_section == 1)
                    {
                        //Get MC Article Infos
                        McArticleInfoWithComments article_info = _uow.Mc_Article.get_mc_article_info_with_comments(s => s.ArticleId == articleid,contract_id).FirstOrDefault();
                        if (article_info != null)
                        {
                            //Get Display state of articles
                            var article_display_states = _uow.Article_Display_State.Find(s => s.ContractId == contract_id &&
                                                                                            s.ArticleId == articleid &&
                                                                                            s.DataSection == 1 &&
                                                                                            s.DelFlag == false).FirstOrDefault();

                            bool isArticleValid = false;
                            bool isArticle_Not_Displayable = article_info.Disp_status == 3;
                            bool isArticle_Not_DisplayableFor_User = false;

                            if (!Screen_ID.Equals("SID1024") & article_display_states != null)
                            {
                                isArticle_Not_DisplayableFor_User = article_display_states.DispStatus == 3;
                            }

                            if (isArticle_Not_Displayable || isArticle_Not_DisplayableFor_User)
                            {
                                isArticleValid = false;
                            }
                            else
                            {
                                List<string> current_Article_theme_ids = themeArticles.Where(s => s.ArticleId == articleid && s.DataSection == 1).Select(s => s.ThemeId).Distinct().ToList();
                                if (Screen_ID.StartsWith("SID0") && !theme_browsing_theme_ids.Intersect(current_Article_theme_ids).Any())
                                {
                                    isArticleValid = false;
                                }
                                else
                                {
                                    //pdf display
                                    if (!mc_text_CanView)
                                    {
                                        isArticleValid = false;
                                        //User does not have permission to view body text
                                        pdfResponse_Models.Data_flag = 2;
                                    }
                                    else
                                    {
                                        isArticleValid = true;
                                    }

                                }
                            }

                            string ArticleText = null;
                            uint[] ArticleTextPdf = null;
                            if (isArticleValid)
                            {
                                if (pdf_flag == 3)
                                {
                                    List<string> article_ids = new List<string>() { articleid };
                                    WAPI014_Response_Model_User outputbodytext = await get_MC_Articles_Text_WAPI014_Async(Mc_API_User_Id, Mc_API_Password, Mc_UserID, article_ids);
                                    if (outputbodytext != null && outputbodytext.OutputResponse != null && outputbodytext.OutputResponse.Count() > 0)
                                    {
                                        ArticleText = outputbodytext.OutputResponse.Where(s => s.Article_return_text != null).Select(s => s.Article_return_text).FirstOrDefault();
                                        if (string.IsNullOrEmpty(ArticleText))
                                        {
                                            //Article does not have any body text
                                            pdfResponse_Models.Data_flag = 3;
                                        }
                                    }
                                    else
                                    {
                                        //Article does not have any body text
                                        pdfResponse_Models.Data_flag = 3;
                                    }
                                }
                                else
                                {
                                    //For article body print
                                    List<string> article_ids = new List<string>() { articleid };
                                    WAPI014_Pdf_Response_Model_User outputbodypdf = await get_MC_Articles_Text_Pdf_WAPI014_Async(Mc_API_User_Id, Mc_API_Password, Mc_UserID, article_ids);
                                    
                                    if (outputbodypdf != null && outputbodypdf.OutputResponse != null && outputbodypdf.OutputResponse.Count() > 0)
                                    {
                                        var article_pdf_byte = outputbodypdf.OutputResponse.Where(s => s.Image_bytes != null).Select(s => s.Image_bytes).FirstOrDefault();
                                        if (article_pdf_byte == null)
                                        {
                                            //Article does not have any body text
                                            pdfResponse_Models.Data_flag = 3;

                                        }
                                        else
                                        {
                                            ArticleTextPdf = Array.ConvertAll(article_pdf_byte, c => (uint)c);
                                        }
                                    }
                                    else
                                    {
                                        //Article does not have any body text
                                        pdfResponse_Models.Data_flag = 3;
                                    }
                                }
                                


                            }

                            if (pdf_flag == 3 & !string.IsNullOrEmpty(ArticleText))
                            {
                                pdfResponse_Models.Data_flag = null;
                                pdfResponse_Models.Pdf_file = null;
                                pdfResponse_Models.Print_flag = 1;
                                pdfResponse_Models.Article_text = ArticleText;
                                pdfResponse_Models.Article_info = null;
                            }
                            else if(pdf_flag == 4 & ArticleTextPdf !=null)
                            {
                                pdfResponse_Models.Data_flag = null;
                                pdfResponse_Models.Pdf_file = ArticleTextPdf;
                                pdfResponse_Models.Print_flag = 1;
                                pdfResponse_Models.Article_text = null;
                                pdfResponse_Models.Article_info = null;
                            }
                            else if (pdfResponse_Models.Data_flag == 3)
                            {
                                
                                pdfResponse_Models.Pdf_file = null;
                                pdfResponse_Models.Print_flag = null;
                                pdfResponse_Models.Article_text = null;
                                pdfResponse_Models.Article_info = null;
                            }
                            else
                            {
                                
                                pdfResponse_Models.Pdf_file = null;
                                pdfResponse_Models.Print_flag = null;
                                pdfResponse_Models.Article_text = null;
                                pdfResponse_Models.Article_info = new List<ArticleInfoItems>() {

                                                    new ArticleInfoItems()
                                                    {
                                                        Article_no = 1,
                                                        Article_id = articleid,
                                                        Data_section = 1,
                                                        Title = article_info.Title,
                                                        Comment = article_info.Comment,
                                                        Mc_article_info = new List<McArticleInfos>()
                                                        {
                                                            new McArticleInfos()
                                                            {
                                                                Media_name = article_info.Media_name,
                                                                Issue_date = article_info.Issue_date,
                                                                Issue_type = article_info.Issue_type,
                                                                Frame_no = article_info.Frame_no,
                                                                Head_no = article_info.Head_no,
                                                                Trace = article_info.Trace,
                                                                Image_count = article_info.Image_count,
                                                                Note = article_info.Note,
                                                                Search_word = themeArticles.Where(s=>s.ArticleId == article_info.Article_id &&
                                                                                                s.DataSection == 1 &&
                                                                                                !string.IsNullOrEmpty(s.SearchWord))
                                                                                                .Select(s=>s.SearchWord).Distinct().ToArray()
                                                            }
                                                        },
                                                        Sc_article_info = null

                                                    }
                                                };
                            }
                        }
                        else
                        {
                            //No article exist in db
                            pdfResponse_Models.Data_flag = 3;
                            pdfResponse_Models.Pdf_file = null;
                            pdfResponse_Models.Print_flag = null;
                            pdfResponse_Models.Article_text = null;
                            pdfResponse_Models.Article_info = null;
                        }
                        

                    }
                    else
                    {
                        //Get SC Article Infos
                        ScArticleInfoWithComments article_info = _uow.Sc_Article.get_sc_article_info_with_comments(s => s.ArticleId == articleid, contract_id).FirstOrDefault();
                        if (article_info != null)
                        {
                            //Get Display state of articles
                            var article_display_states = _uow.Article_Display_State.Find(s => s.ContractId == contract_id &&
                                                                                            s.ArticleId == articleid &&
                                                                                            s.DataSection == 2 &&
                                                                                            s.DelFlag == false).FirstOrDefault();

                            bool isArticleValid = false;
                            bool isArticle_Not_Displayable = article_info.Disp_status == 3;
                            bool isArticle_Not_DisplayableFor_User = false;

                            if (!Screen_ID.Equals("SID1024") & article_display_states != null)
                            {
                                isArticle_Not_DisplayableFor_User = article_display_states.DispStatus == 3;
                            }

                            if (isArticle_Not_Displayable || isArticle_Not_DisplayableFor_User)
                            {
                                isArticleValid = false;
                            }
                            else
                            {
                                List<string> current_Article_theme_ids = themeArticles.Where(s => s.ArticleId == articleid && s.DataSection == 2).Select(s => s.ThemeId).Distinct().ToList();
                                if (Screen_ID.StartsWith("SID0") && !theme_browsing_theme_ids.Intersect(current_Article_theme_ids).Any())
                                {
                                    isArticleValid = false;
                                }
                                else
                                {
                                    //text display
                                    isArticleValid = true;

                                }
                            }

                            if (isArticleValid && !string.IsNullOrEmpty(article_info.Article_text))
                            {
                                pdfResponse_Models.Data_flag = null;
                                pdfResponse_Models.Pdf_file = null;
                                pdfResponse_Models.Print_flag = 1;
                                pdfResponse_Models.Article_text = article_info.Article_text;
                                pdfResponse_Models.Article_info = null;


                            }           
                            else
                            {
                                pdfResponse_Models.Data_flag = 3;
                                pdfResponse_Models.Pdf_file = null;
                                pdfResponse_Models.Print_flag = null;
                                pdfResponse_Models.Article_text = null;
                                pdfResponse_Models.Article_info = new List<ArticleInfoItems>() {

                                    new ArticleInfoItems()
                                    {
                                        Article_no = 1,
                                        Article_id = articleid,
                                        Data_section = 1,
                                        Title = article_info.Title,
                                        Comment = article_info.Comment,
                                        Mc_article_info = null,
                                        Sc_article_info =  new List<ScArticleInfos>()
                                        {
                                            new ScArticleInfos()
                                            {
                                                Media_name = article_info.Media_name,
                                                Issue_date = article_info.Issue_date,
                                                Issue_type = article_info.Issue_type,
                                                Frame_no = article_info.Frame_no,
                                                Char_count = article_info.Char_count,
                                                Search_word = themeArticles.Where(s=>s.ArticleId == article_info.Article_id &&
                                                                                s.DataSection == 2 &&
                                                                                !string.IsNullOrEmpty(s.SearchWord))
                                                                                .Select(s=>s.SearchWord).Distinct().ToArray()

                                            }
                                        }

                                    }
                                };
                            }
                        }
                        else
                        {
                            //No article in DB
                            pdfResponse_Models.Data_flag = 3;
                            pdfResponse_Models.Pdf_file = null;
                            pdfResponse_Models.Print_flag = null;
                            pdfResponse_Models.Article_text = null;
                            pdfResponse_Models.Article_info = null;
                        }
                        
                        


                    }

                }
            }
            catch(Exception e)
            {
                pdfResponse_Models = null;
            }
           


            return pdfResponse_Models;
        }

        /// <summary>
        /// Returns theme ids based on theme browsing
        /// Theme_browsing =true,themes under the theme group linked to your user group.
        /// Theme_browsing =false,all themes of this contractor
        /// </summary>
        /// <param name="theme_browsing"></param>
        /// <param name="user_group_id"></param>
        /// <param name="contract_id"></param>
        /// <returns></returns>
        private List<string> getThemeBrowsing_ThemeIDs(bool theme_browsing,int user_group_id,int contract_id)
        {
            List<string> theme_browsing_themeids = new List<string>();
           
         


            if (theme_browsing == false)
            {
                //Get all the theme ids not mapped to any of the theme group in the current contract id
                //List<string> theme_ids_not_mapped_to_theme_groups = theme_ids.Except(theme_group_theme_ids).ToList();
                //theme_browsing_themeids.AddRange(theme_group_theme_ids_usergroup);
                //theme_browsing_themeids.AddRange(theme_ids_not_mapped_to_theme_groups);

                //all themes of this contractor
                //Get all theme ids for the contract id
                List<string> theme_ids = _uow.Theme.Find(s => s.ContractId == contract_id && s.DelFlag == false)
                                                    .Select(s => s.ThemeId).Distinct().ToList();
                theme_browsing_themeids = theme_ids;


            }
            else
            {
                //themes under the theme group linked to your user group.

                //Get all theme group users
                var theme_groups_users = _uow.Theme_Group_User.Find(f => f.ContractId == contract_id && f.DelFlag == false).ToList();

                //Get all theme group items
                var theme_group_items = _uow.Theme_Group_Item.Find(s => s.ContractId == contract_id && s.DelFlag == false).ToList();


                //Get all theme group ids mapped to the usergroup id
                List<int> themeGroupIds_By_Usergroup = theme_groups_users.Where(f => f.UserGroupId == user_group_id).Select(f => f.ThemeGroupId).Distinct().ToList();
                //Get all theme ids mapped to the theme group ids of the current user group
                List<string> theme_group_theme_ids_usergroup = theme_group_items.Where(s => themeGroupIds_By_Usergroup.Contains(s.ThemeGroupId))
                                                                    .Select(s => s.ThemeId).Distinct().ToList();

                theme_browsing_themeids = theme_group_theme_ids_usergroup;
            }


            return theme_browsing_themeids.Distinct().ToList();
        }
       
        private async Task<WAPI008_Response_Model_User> get_MC_Articles_Pdf_WAPI008_Async(string API_User_Id,string API_Password,string Mc_UserID,List<string> article_ids)
        {
            WAPI008_Response_Model_User bID0018_Response_Model = new WAPI008_Response_Model_User();

            List<WAPI008_Output_ViewModel_user> bID0018_Output_ViewModels = new List<WAPI008_Output_ViewModel_user>();

            try
            {
 
                for (int i = 0; i < article_ids.Count(); i++)
                {
                    IN_WAPI008_PDF in_wapi008 = new IN_WAPI008_PDF()
                    {
                        ac = article_ids[i],
                        dmsid = API_User_Id,
                        password = API_Password,
                        mcuserid = Mc_UserID
                    };
                    HttpResponseMessage pdfresponse = await EIF_Network_Handler.GetDataForWAPI(in_wapi008, EIF_URL_Config.MC_WAPI008_URL);
                    if (pdfresponse.IsSuccessStatusCode)
                    {
                        string pdfresponseString = await pdfresponse.Content.ReadAsStringAsync();
                        OP_MC_WAPI008_PDF oP_MC_WAPI008 = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_MC_WAPI008_PDF>(pdfresponseString);
                        if (typeof(OP_MC_WAPI008_PDF).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                        {
                            (oP_MC_WAPI008 as I_OP_BaseModel).SetUpCommonResponse();
                        }


                        if (oP_MC_WAPI008.ExternalAPIResonse != null && oP_MC_WAPI008.ExternalAPIResonse.IsSuccessResponse)
                        {
                            WAPI008_Output_ViewModel_user bID0018_Output_ViewModel = new WAPI008_Output_ViewModel_user()
                            {
                                Image_filename = article_ids[i],
                                Image_bytes = Convert.FromBase64String(oP_MC_WAPI008.pdf)
                            };

                            bID0018_Output_ViewModels.Add(bID0018_Output_ViewModel);
                        }
                        else
                        {
                            // if failer response return, it always return below failer response.
                            // Code review change (CRC): GT
                            WAPI008_Output_ViewModel_user bID0018_Output_ViewModel = new WAPI008_Output_ViewModel_user()
                            {
                                Image_filename = article_ids[i],
                                Image_bytes = null
                            };
                            bID0018_Output_ViewModels.Add(bID0018_Output_ViewModel);
                        }
                        
                    }
                    else
                    {
                        WAPI008_Output_ViewModel_user bID0018_Output_ViewModel = new WAPI008_Output_ViewModel_user()
                        {
                            Image_filename = article_ids[i],
                            Image_bytes = null
                        };
                        bID0018_Output_ViewModels.Add(bID0018_Output_ViewModel);
                    }

                }

                bID0018_Response_Model.OutputResponse = bID0018_Output_ViewModels;
            }
            catch(Exception e)
            {
                bID0018_Response_Model.OutputResponse = null;
            }
                        
            return bID0018_Response_Model;
        }

        private async Task<ATCD017_Response_Model_User> get_SC_Articles_Pdf_ATCD017_Async(string API_User_Id, string API_Password, Dictionary<string,string> article_ids)
        {
            ATCD017_Response_Model_User bID0019_Response_Model = new ATCD017_Response_Model_User();

            List<ATCD017_Output_ViewModel_User> bID0019_Output_ViewModels = new List<ATCD017_Output_ViewModel_User>();

            try
            {

                //Login to SC
                IN_CMN0F13_LoginModel ScLogin = new IN_CMN0F13_LoginModel()
                {
                    actionId = "CMN0F13",
                    userId = API_User_Id,
                    password = API_Password

                };
                HttpResponseMessage login_response = await EIF_Network_Handler.PostData(ScLogin, EIF_URL_Config.SC_BASE_URL);
                if (login_response.StatusCode == System.Net.HttpStatusCode.OK)
                {
                    string loginresString = await login_response.Content.ReadAsStringAsync();
                    OP_CMN0F21_LogoutModel loginResponseData = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_CMN0F21_LogoutModel>(loginresString);
                    if (typeof(OP_CMN0F21_LogoutModel).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                    {
                        (loginResponseData as I_OP_BaseModel).SetUpCommonResponse();
                    }
                    if (loginResponseData.ExternalAPIResonse != null && loginResponseData.ExternalAPIResonse.IsSuccessResponse)
                    {
                        
                        foreach(var article in article_ids)
                        {
                            string article_id = article.Key;
                            string pdf_key = article.Value;
                            IN_ATCD017_ElectronicArticlePDFModel iN_ATCD017_ElectronicArticlePDF = new IN_ATCD017_ElectronicArticlePDFModel()
                            {
                                keyPdf = pdf_key
                            };
                            HttpResponseMessage ATCD017_res = await EIF_Network_Handler.PostDataWithCookieLogin(iN_ATCD017_ElectronicArticlePDF, EIF_URL_Config.SC_BASE_URL, login_response);
                            if (ATCD017_res.StatusCode == System.Net.HttpStatusCode.OK)
                            {
                                MemoryStream MS_pdf = new MemoryStream();
                                var Result_pdf = ATCD017_res.Content.ReadAsStreamAsync().Result;

                                Result_pdf.CopyTo(MS_pdf);
                                MS_pdf.Position = 0;

                                ATCD017_Output_ViewModel_User bID0019_Output_ViewModel = new ATCD017_Output_ViewModel_User()
                                {
                                    Image_filename = article_id,
                                    Image_bytes = MS_pdf.ToArray()
                                };

                                bID0019_Output_ViewModels.Add(bID0019_Output_ViewModel);
                            }
                            else
                            {
                                ATCD017_Output_ViewModel_User bID0019_Output_ViewModel = new ATCD017_Output_ViewModel_User()
                                {
                                    Image_filename = article_id,
                                    Image_bytes = null
                                };

                                bID0019_Output_ViewModels.Add(bID0019_Output_ViewModel);
                            }
                        }

                        //Calling Logout api
                        IN_CMN0F21_LogoutModel _LogoutModel = new IN_CMN0F21_LogoutModel()
                        {
                            actionId = "CMN0F21"
                        };
                        HttpResponseMessage _logoutresponse = await EIF_Network_Handler.PostDataWithCookieLogin(_LogoutModel, EIF_URL_Config.SC_BASE_URL, login_response);

                        bID0019_Response_Model.OutputResponse = bID0019_Output_ViewModels;
                    }
                    else
                    {
                        bID0019_Response_Model.OutputResponse = null;
                    }
                }
    
            }
            catch (Exception e)
            {
                bID0019_Response_Model.OutputResponse = null;
            }

            return bID0019_Response_Model;
        }

        private async Task<WAPI014_Response_Model_User> get_MC_Articles_Text_WAPI014_Async(string API_User_Id, string API_Password, string Mc_UserID, List<string> article_ids)
        {
            WAPI014_Response_Model_User bID0022_Response_Model = new WAPI014_Response_Model_User();

            List<WAPI014_Output_ViewModel_User> bID0022_Output_ViewModels = new List<WAPI014_Output_ViewModel_User>();

            try
            {

                for (int i = 0; i < article_ids.Count(); i++)
                {
                    IN_WAPI014_ArticleText wapi014_input = new IN_WAPI014_ArticleText()
                    {
                        ac = article_ids[i],
                        dmsid = API_User_Id,
                        password = API_Password,
                        mcuserid = Mc_UserID,
                        responsetype = "text"
                    };
                    HttpResponseMessage textresponse = await EIF_Network_Handler.GetDataForWAPI(wapi014_input, EIF_URL_Config.MC_WAPI014_URL);
                    if (textresponse.IsSuccessStatusCode)
                    {
                        string textresponseString = await textresponse.Content.ReadAsStringAsync();
                        OP_MC_WAPI014_ArticleText oP_MC_WAPI014 = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_MC_WAPI014_ArticleText>(textresponseString);
                        if (typeof(OP_MC_WAPI014_ArticleText).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                        {
                            (oP_MC_WAPI014 as I_OP_BaseModel).SetUpCommonResponse();
                        }


                        if (oP_MC_WAPI014.ExternalAPIResonse != null && oP_MC_WAPI014.ExternalAPIResonse.IsSuccessResponse)
                        {
                            WAPI014_Output_ViewModel_User bID0022_Output_ViewModel = new WAPI014_Output_ViewModel_User()
                            {
                                Article_id = article_ids[i],
                                Article_return_text = oP_MC_WAPI014.text
                            };

                            bID0022_Output_ViewModels.Add(bID0022_Output_ViewModel);
                        }
                        else
                        {
                            WAPI014_Output_ViewModel_User bID0022_Output_ViewModel = new WAPI014_Output_ViewModel_User()
                            {
                                Article_id = article_ids[i],
                                Article_return_text = null
                            };

                            bID0022_Output_ViewModels.Add(bID0022_Output_ViewModel);
                        }

                    }
                    else
                    {
                        WAPI014_Output_ViewModel_User bID0022_Output_ViewModel = new WAPI014_Output_ViewModel_User()
                        {
                            Article_id = article_ids[i],
                            Article_return_text = null
                        };

                        bID0022_Output_ViewModels.Add(bID0022_Output_ViewModel);
                    }

                }

                bID0022_Response_Model.OutputResponse = bID0022_Output_ViewModels;
            }
            catch (Exception e)
            {
                bID0022_Response_Model.OutputResponse = null;
            }

            return bID0022_Response_Model;
        }

        private async Task<WAPI014_Pdf_Response_Model_User> get_MC_Articles_Text_Pdf_WAPI014_Async(string API_User_Id, string API_Password, string Mc_UserID, List<string> article_ids)
        {
            WAPI014_Pdf_Response_Model_User wapi014_pdf_response = new WAPI014_Pdf_Response_Model_User();

            List<WAPI014_Pdf_Output_ViewModel_User> wapi014_pdf_Output_ViewModels = new List<WAPI014_Pdf_Output_ViewModel_User>();

            try
            {

                for (int i = 0; i < article_ids.Count(); i++)
                {
                    IN_WAPI014_ArticleText wapi014_input = new IN_WAPI014_ArticleText()
                    {
                        ac = article_ids[i],
                        dmsid = API_User_Id,
                        password = API_Password,
                        mcuserid = Mc_UserID,
                        responsetype = "pdf"
                    };
                    HttpResponseMessage textresponse = await EIF_Network_Handler.GetDataForWAPI(wapi014_input, EIF_URL_Config.MC_WAPI014_URL);
                    if (textresponse.IsSuccessStatusCode)
                    {
                        string textresponseString = await textresponse.Content.ReadAsStringAsync();
                        OP_MC_WAPI014_ArticleText oP_MC_WAPI014 = CommonLibrary.Helper.CommonUtil.DeserializeXMLToObject<OP_MC_WAPI014_ArticleText>(textresponseString);
                        if (typeof(OP_MC_WAPI014_ArticleText).GetInterfaces().Contains(typeof(I_OP_BaseModel)))
                        {
                            (oP_MC_WAPI014 as I_OP_BaseModel).SetUpCommonResponse();
                        }


                        if (oP_MC_WAPI014.ExternalAPIResonse != null && oP_MC_WAPI014.ExternalAPIResonse.IsSuccessResponse)
                        {
                            WAPI014_Pdf_Output_ViewModel_User wapi014_pdf_Output_ViewModel = new WAPI014_Pdf_Output_ViewModel_User()
                            {
                                Article_id = article_ids[i],
                                Image_bytes = Convert.FromBase64String(oP_MC_WAPI014.pdf)
                            };

                            wapi014_pdf_Output_ViewModels.Add(wapi014_pdf_Output_ViewModel);
                        }
                        else
                        {
                            WAPI014_Pdf_Output_ViewModel_User wapi014_pdf_Output_ViewModel = new WAPI014_Pdf_Output_ViewModel_User()
                            {
                                Article_id = article_ids[i],
                                Image_bytes = null
                            };

                            wapi014_pdf_Output_ViewModels.Add(wapi014_pdf_Output_ViewModel);
                        }

                    }
                    else
                    {
                        WAPI014_Pdf_Output_ViewModel_User wapi014_pdf_Output_ViewModel = new WAPI014_Pdf_Output_ViewModel_User()
                        {
                            Article_id = article_ids[i],
                            Image_bytes = null
                        };

                        wapi014_pdf_Output_ViewModels.Add(wapi014_pdf_Output_ViewModel);
                    }

                }

                wapi014_pdf_response.OutputResponse = wapi014_pdf_Output_ViewModels;
            }
            catch (Exception e)
            {
                wapi014_pdf_response.OutputResponse = null;
            }

            return wapi014_pdf_response;
        }

        private byte[] merge_pdfs(List<byte[]> byte_pdfs)
        {
            byte[] bytes_merge = null;
            try
            {
                using (MemoryStream ms = new MemoryStream())
                {
                    //Using the ItextSharp Document
                    using (Document document = new Document(PageSize.A4))
                    {
                        //Using the ITextSharp PdfCopy to create a PDF document in the memory stream
                        using (PdfCopy copy = new PdfCopy(document, ms))
                        {
                            //Open the document before any changes can be made.
                            document.Open();
                            
                            //Loop through each file
                            foreach (var pdf in byte_pdfs)
                            {
                                // Convert the BinaryStream of the file to a ITexSharp PdfReader
                                PdfReader reader = new PdfReader(pdf);
                                int n = reader.NumberOfPages;
                                //Loop through each page in the current PDF file
                                for (int page = 0; page < n;)
                                {
                                    //Import the page to the PDF document in the memory stream.
                                    copy.AddPage(copy.GetImportedPage(reader, ++page));
                                }
                            }
                            
                        }
                        
                        
                        //Convert the final memory stream into a byte array.
                        bytes_merge = ms.ToArray();
                        var merge_base64 = Convert.ToBase64String(bytes_merge);
                        document.Close();
                    }
 
                }
            }
            catch(Exception e)
            {

            }
            return bytes_merge;
        }

        private byte[] add_pagenumber_pdfs(byte[] byte_pdf)
        {
            byte[] bytes_pagenumber_pdf = null;
            try
            {
                Font blackFont = FontFactory.GetFont("Arial", 12, Font.NORMAL, BaseColor.BLACK);
                using (MemoryStream stream = new MemoryStream())
                {
                    PdfReader reader = new PdfReader(byte_pdf);
                    using (PdfStamper stamper = new PdfStamper(reader, stream))
                    {
                        //stamper.Writer.CloseStream = false;
                        int pages = reader.NumberOfPages;
                        for (int i = 1; i <= pages; i++)
                        {
                            ColumnText.ShowTextAligned(stamper.GetUnderContent(i), Element.ALIGN_RIGHT, new Phrase(i.ToString(), blackFont), 568f, 15f, 0);
                        }

                        //stamper.Close();
                    }
                    
                    bytes_pagenumber_pdf = stream.ToArray();

                }
                string test_base64 = Convert.ToBase64String(bytes_pagenumber_pdf);
            
            }
            catch (Exception e)
            {

            }
            return bytes_pagenumber_pdf;
        }

        private byte[] encrypt_pdf(byte[] byte_pdf,string user_password,string admin_password,int permissions)
        {
            byte[] bytes_encrypt_pdf = null;
            try
            {
                
                using (MemoryStream stream = new MemoryStream())
                {
                    PdfReader reader = new PdfReader(byte_pdf);
                    using (PdfStamper stamper = new PdfStamper(reader, stream))
                    {
                        System.Text.UTF8Encoding Encoding = new System.Text.UTF8Encoding();
                        var up = user_password != null ? Encoding.GetBytes(user_password) : null;
                        stamper.SetEncryption(up, Encoding.GetBytes(admin_password), permissions, PdfWriter.STRENGTH128BITS);
                        stamper.Close();
                    }
                    bytes_encrypt_pdf = stream.ToArray();

                }
                string test_base64 = Convert.ToBase64String(bytes_encrypt_pdf);

            }
            catch (Exception e)
            {

            }
            return bytes_encrypt_pdf;
        }

    }
}
